import csv
import json
import multiprocessing
import operator
import os
import sys
import warnings
from functools import reduce

import django
from django.db import transaction
from django.db.models import F, Func, IntegerField, Q
from django.db.models.functions import Length

from pbn_api.integrator import istarmap  # noqa

from bpp.const import PBN_MIN_ROK

try:
    django.setup()
except RuntimeError:
    pass

from import_common.core import matchuj_autora, matchuj_wydawce
from import_common.normalization import (
    normalize_doi,
    normalize_isbn,
    normalize_tytul_publikacji,
)
from pbn_api.client import PBNClient
from pbn_api.const import ACTIVE, DELETED
from pbn_api.exceptions import (
    HttpException,
    SameDataUploadedRecently,
    StatementDeletionError,
    WillNotExportError,
)
from pbn_api.integrator.threaded_page_getter import (
    ThreadedMongoDBSaver,
    threaded_page_getter,
)
from pbn_api.models import (
    Conference,
    Country,
    Institution,
    Journal,
    Language,
    OswiadczenieInstytucji,
    Publication,
    PublikacjaInstytucji,
    Publisher,
    Scientist,
    SentData,
)

from django.contrib.postgres.search import TrigramSimilarity

from bpp.models import (
    Autor,
    Autor_Dyscyplina,
    Jednostka,
    Jezyk,
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Rekord,
    Uczelnia,
    Wydawca,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Ciagle_Autor,
    Wydawnictwo_Zwarte,
    Wydawnictwo_Zwarte_Autor,
    Zrodlo,
)
from bpp.util import pbar


def integruj_jezyki(client):
    for remote_lang in client.get_languages():

        try:
            lang = Language.objects.get(code=remote_lang["code"])
        except Language.DoesNotExist:
            lang = Language.objects.create(
                code=remote_lang["code"], language=remote_lang["language"]
            )

        if remote_lang["language"] != lang.language:
            lang.language = remote_lang["language"]
            lang.save()

    # Ustaw odpowiedniki w PBN dla jęyzków z bazy danych:
    for elem in Language.objects.all():
        try:
            qry = Q(skrot__istartswith=elem.language.get("639-2")) | Q(
                skrot__istartswith=elem.code
            )
            if elem.language.get("639-1"):
                qry |= Q(skrot__istartswith=elem.language["639-1"])

            jezyk = Jezyk.objects.get(qry)
        except Jezyk.DoesNotExist:
            if elem.language.get("pl") is not None:
                try:
                    jezyk = Jezyk.objects.get(
                        nazwa__istartswith=elem.language.get("pl")
                    )
                except Jezyk.DoesNotExist:
                    # warnings.warn(f"Brak jezyka po stronie BPP: {elem}")
                    continue
            else:
                # warnings.warn(f"Brak jezyka po stronie BPP: {elem}")
                continue

        if jezyk.pbn_uid_id is None:
            jezyk.pbn_uid = elem
            jezyk.save()


def integruj_kraje(client):
    for remote_country in client.get_countries():

        try:
            c = Country.objects.get(code=remote_country["code"])
        except Country.DoesNotExist:
            c = Country.objects.create(
                code=remote_country["code"], description=remote_country["description"]
            )
            continue

        if remote_country["description"] != c.description:
            c.description = remote_country["description"]
            c.save()


@transaction.atomic
def zapisz_mongodb(elem, klass, client=None, **extra):
    defaults = dict(
        status=elem["status"],
        verificationLevel=elem["verificationLevel"],
        verified=elem["verified"],
        versions=elem["versions"],
        **extra,
    )

    existing = klass.objects.select_for_update().filter(pk=elem["mongoId"])

    created = False
    try:
        v = existing.get()
    except klass.DoesNotExist:
        v = klass.objects.create(pk=elem["mongoId"], **defaults)
        created = True

    if not created:
        needs_saving = False

        for key in extra:
            if getattr(v, key) != extra.get(key):
                setattr(v, key, extra.get(key))
                needs_saving = True

        if elem["versions"] != v.versions:
            v.versions = elem["versions"]
            needs_saving = True

        if needs_saving:
            v.save()

    return v


def ensure_publication_exists(client, publicationId):
    try:
        publicationId = Publication.objects.get(pk=publicationId)
    except Publication.DoesNotExist:
        zapisz_mongodb(
            client.get_publication_by_id(publicationId), Publication, client=client
        )


def ensure_person_exists(client: PBNClient, personId):
    try:
        personId = Scientist.objects.get(pk=personId)
    except Scientist.DoesNotExist:
        zapisz_mongodb(client.get_person_by_id(personId), Scientist, client=client)


def ensure_institution_exists(client: PBNClient, institutionId):
    try:
        institutionId = Institution.objects.get(pk=institutionId)
    except Institution.DoesNotExist:
        zapisz_mongodb(
            client.get_publication_by_id(institutionId), Institution, client=client
        )


def zapisz_publikacje_instytucji(elem, klass, client=None, **extra):
    try:
        ensure_publication_exists(client, elem["publicationId"])
    except HttpException as e:
        if e.status_code == 500:
            print(
                f"Podczas zapisywania publikacji instytucji, dla {elem['publicationId']} serwer "
                f"PBN zwrócił bład 500. Publikacja instytucji może zostac nie zapisana poprawnie. Dump danych "
                f"publikacji: {elem}"
            )
            return

    rec, _ign = PublikacjaInstytucji.objects.get_or_create(
        institutionId_id=elem["institutionId"],
        publicationId_id=elem["publicationId"],
        insPersonId_id=elem["insPersonId"],
    )

    changed = False
    for key in (
        "publicationVersion",
        "publicationYear",
        "publicationType",
        "userType",
        "snapshot",
    ):
        current = getattr(rec, key)
        server = elem.get(key)
        if current != server:
            setattr(rec, key, server)
            changed = True
    if changed:
        rec.save()


def zapisz_oswiadczenie_instytucji(elem, klass, client=None, **extra):
    # {'id': '121339', 'institutionId': '5e70918b878c28a04737de51',
    # 'personId': '5e709330878c28a0473a4a0e', 'publicationId': '5ebff589ad49b31ccec25471',
    # 'addedTimestamp': '2020.05.16', 'area': '304', 'inOrcid': False, 'type': 'AUTHOR'}
    for var in "addedTimestamp", "statedTimestamp":
        if elem.get(var):
            elem[var] = elem[var].replace(".", "-")

    try:
        ensure_publication_exists(client, elem["publicationId"])
    except HttpException as e:
        if e.status_code == 500:
            print(
                f"Podczas próby pobrania danych o nie-istniejącej obecnie po naszej stronie publikacji o id"
                f" {elem['publicationId']} z PBNu, wystąpił błąd wewnętrzny serwera po ich stronie. Dane "
                f"dotyczące oświadczeń z tej publikacji nie zostały zapisane. "
            )
            return

    try:
        ensure_institution_exists(client, elem["institutionId"])
    except HttpException as e:
        if e.status_code == 500:
            print(
                f"Podczas próby pobrania danych o nie-istniejącej obecnie po naszej stronie publikacji o id"
                f" {elem['publicationId']} z PBNu, wystąpił błąd wewnętrzny serwera po ich stronie. Dane "
                f"dotyczące oświadczeń z tej publikacji nie zostały zapisane. "
            )
            return

    try:
        ensure_person_exists(client, elem["personId"])
    except HttpException as e:
        if e.status_code == 500:
            print(
                f"Podczas próby pobrania danych o nie-istniejącej obecnie po naszej stronie publikacji o id"
                f" {elem['publicationId']} z PBNu, wystąpił błąd wewnętrzny serwera po ich stronie. Dane "
                f"dotyczące oświadczeń z tej publikacji nie zostały zapisane. "
            )
            return

    for key in "institution", "person", "publication":
        elem[f"{key}Id_id"] = elem[f"{key}Id"]
        del elem[f"{key}Id"]

    OswiadczenieInstytucji.objects.update_or_create(id=elem["id"], defaults=elem)


@transaction.atomic
def pobierz_mongodb(
    elems,
    klass,
    pbar_label="pobierz_mongodb",
    fun=zapisz_mongodb,
    client=None,
    disable_progress_bar=False,
):
    if hasattr(elems, "total_elements"):
        count = elems.total_elements
    elif hasattr(elems, "__len__"):
        count = len(elems)

    for elem in pbar(
        elems, count, pbar_label, disable_progress_bar=disable_progress_bar
    ):
        fun(elem, klass, client=client)


def pobierz_instytucje(client: PBNClient):
    for status in [ACTIVE, DELETED]:
        pobierz_mongodb(
            client.get_institutions(status=status, page_size=1000), Institution
        )


def integruj_uczelnie():
    uczelnia = Uczelnia.objects.get_default()

    try:
        u = Institution.objects.get(
            versions__contains=[{"current": True, "object": {"name": uczelnia.nazwa}}]
        )
    except Institution.DoesNotExist:
        raise Exception(f"Nie umiem dopasowac uczelni po nazwie: {uczelnia.nazwa}")

    if uczelnia.pbn_uid_id != u.mongoId:
        uczelnia.pbn_uid = u
        uczelnia.save()


def integruj_instytucje():
    uczelnia = Uczelnia.objects.get_default()
    assert uczelnia.pbn_uid_id

    for j in Jednostka.objects.filter(skupia_pracownikow=True):
        try:
            u = Institution.objects.get(
                versions__contains=[
                    {
                        "current": True,
                        "object": {"name": j.nazwa, "parentId": uczelnia.pbn_uid_id},
                    }
                ]
            )
        except Institution.MultipleObjectsReturned:
            print(f"x*X Liczne dopasowania dla jednostki: {j}")
            continue

        except Institution.DoesNotExist:
            print(f"XXX Brak dopasowania dla jednostki: {j}")
            continue

        if j.pbn_uid_id != u.mongoId:
            j.pbn_uid_id = u.mongoId
            j.save()


def pobierz_konferencje(client: PBNClient):
    pobierz_mongodb(
        client.get_conferences_mnisw(page_size=1000),
        Conference,
        pbar_label="pobierz_konferencje",
    )


class ZrodlaGetter(ThreadedMongoDBSaver):
    pbn_api_klass = Journal


def pobierz_zrodla(client: PBNClient):
    for status in [DELETED, ACTIVE]:
        data = client.get_journals(status=status, page_size=500)
        threaded_page_getter(
            client, data, klass=ZrodlaGetter, label=f"poboerz_zrodla_{status}"
        )


class PublisherGetter(ThreadedMongoDBSaver):
    pbn_api_klass = Publisher


def pobierz_wydawcow_mnisw(client: PBNClient):
    data = client.get_publishers_mnisw(page_size=1000)
    threaded_page_getter(
        client, data, klass=PublisherGetter, label="poboerz_wydawcow_mnisw"
    )


def pobierz_wydawcow_wszystkich(client: PBNClient):
    data = client.get_publishers(page_size=1000)
    threaded_page_getter(
        client, data, klass=PublisherGetter, label="poboerz_wydawcow_wszystkich"
    )

    # pobierz_mongodb(
    #     client.get_publishers(page_size=1000),
    #     Publisher,
    #     pbar_label="pobierz_wydawcow_wszystkich",
    # )


def pbn_file_path(db, current_page, status):
    return f"pbn_json_data/{db}_offline_{status}_{current_page}.json"


def wait_for_results(pool, results, label="Progress..."):
    for elem in pbar(results, count=len(results), label=label):
        elem.get()
    pool.close()
    pool.join()


def _single_unit_offline(data, db, current_page, status):
    f = open(pbn_file_path(db, current_page, status), "w")
    f.write(json.dumps(data.fetch_page(current_page)))
    f.close()


def _pobierz_offline(fun, db):
    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()

    p = initialize_pool()

    for status in [ACTIVE, DELETED]:

        data = fun(status=status, page_size=10000)

        params = [
            (data, db, current_page, status)
            for current_page in range(0, data.total_pages)
        ]

        for _ in pbar(
            p.istarmap(_single_unit_offline, params),
            label=f"pobierz_offline_{db}_{status}",
            count=len(params),
        ):
            pass

    # wait_for_results(p, results, label=f"pobierz_offline {db}")
    p.close()
    p.join()


def pobierz_ludzi_offline(client: PBNClient):
    return _pobierz_offline(client.get_people, "people")


def pobierz_prace_offline(client: PBNClient):
    return _pobierz_offline(client.get_publications, "publications")


def _single_unit_wgraj(current_page, status, db, model):
    fn = pbn_file_path(db, current_page, status)
    if os.path.exists(fn):
        data = open(fn).read()
        if data:
            dane = json.loads(data)
            if dane:
                with transaction.atomic():
                    pobierz_mongodb(dane, model, disable_progress_bar=True)


def _bede_uzywal_bazy_danych_z_multiprocessing_z_django():
    from django.db import close_old_connections, connections

    close_old_connections()
    connections.close_all()


def _wgraj_z_offline_do_bazy(db, model):
    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()

    p = initialize_pool()

    for status in [ACTIVE, DELETED]:
        params = [(current_page, status, db, model) for current_page in range(1500)]

        for _ in pbar(
            p.istarmap(_single_unit_wgraj, params),
            label=f"wgraj_offline_{db}_{status}",
            count=len(params),
        ):
            pass

    p.close()
    p.join()


def wgraj_ludzi_z_offline_do_bazy():
    return _wgraj_z_offline_do_bazy("people", Scientist)


def wgraj_prace_z_offline_do_bazy():
    return _wgraj_z_offline_do_bazy("publications", Publication)


def pobierz_prace(client: PBNClient):
    pobierz_mongodb(
        client.get_publications(status=ACTIVE, page_size=5000),
        Publication,
        pbar_label="pobierz_aktywne_prace",
    )


def pobierz_publikacje_z_instytucji(client: PBNClient):
    pobierz_mongodb(
        client.get_institution_publications(page_size=2000),
        None,
        fun=zapisz_publikacje_instytucji,
        client=client,
        pbar_label="pobierz_publikacje_instytucji",
    )


def pobierz_oswiadczenia_z_instytucji(client: PBNClient):
    pobierz_mongodb(
        client.get_institution_statements(page_size=1000),
        None,
        fun=zapisz_oswiadczenie_instytucji,
        client=client,
        pbar_label="oswiadczenia instytucji",
    )


def _pobierz_prace_po_elemencie(
    client: PBNClient, element, nd, matchuj=True, value_from_rec=None, **kw
):
    ret = []
    base_kw = {element: nd}
    base_kw.update(kw)

    for elem in client.search_publications(**base_kw):
        publication = zapisz_mongodb(
            client.get_publication_by_id(elem["mongoId"]), Publication
        )

        p = publication.matchuj_do_rekordu_bpp()
        if p is None:
            print(
                f"XXX mimo pobrania pracy po {element.upper()} {nd}, zwrotnie NIE pasuje ona do pracy w BPP "
                f"-- rok {publication.year} lub tytul {publication.title} nie daja sie dopasowac. Blad w "
                f"zapisie {element.upper()}? Niepoprawne {element.upper()}? Oryginalny obiekt: {value_from_rec}"
            )
            continue

        p = p.original

        if p in ret:
            continue

        if p.pbn_uid_id == publication.mongoId:
            # Juz zmatchowany
            continue

        # Co w sytuacji p.pbn_uid_id != publication.mongoId? Rekord zmatchowany z innym, teraz pasuje do innego?
        # Nic. Na ten moment nadpisujemy to po cichu (2.09.2021), jest to moja swiadoma decyzja (mpasternak)
        # poniewaz z tysiaca komunikatow o tej sytuacji nic nie wynikalo a zazwyczaj wychodzilo na to, ze rekordy
        # byly niepoprawnie zmatchowane.

        p.pbn_uid_id = publication.mongoId
        p.save(update_fields=["pbn_uid_id"])
        ret.append(p)

    return ret


def pobierz_prace_po_doi(client: PBNClient):
    dois = set()
    for praca in pbar(
        Rekord.objects.all()
        .exclude(doi=None)
        .exclude(doi="")
        .filter(pbn_uid_id=None)
        .filter(rok__gte=PBN_MIN_ROK),
        label="pobierz_prace_po_doi",
    ):
        nd = normalize_doi(praca.doi)
        dois.add(nd)

    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()

    p = initialize_pool()

    results = []

    for doi in dois:
        results.append(
            p.apply_async(
                _pobierz_prace_po_elemencie,
                args=(
                    client,
                    "doi",
                    doi,
                ),
            )
        )

    wait_for_results(p, results)


def pobierz_prace_po_isbn(client: PBNClient):
    isbns = set()
    for praca in pbar(
        Rekord.objects.all()
        .exclude(isbn=None)
        .exclude(isbn="")
        .filter(pbn_uid_id=None)
        .filter(rok__gte=PBN_MIN_ROK)
        .filter(wydawnictwo_nadrzedne_id=None),
        label="pobierz_prace_po_isbn",
    ):
        nd = normalize_isbn(praca.isbn)
        isbns.add(nd)

    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()

    p = initialize_pool()

    results = []

    for isbn in isbns:
        results.append(
            p.apply_async(
                _pobierz_prace_po_elemencie,
                args=(
                    client,
                    "isbn",
                    isbn,
                ),
            )
        )

    wait_for_results(p, results)


def _single_unit_ludzie_z_uczelni(client, personId):
    scientist = client.get_person_by_id(personId)
    zapisz_mongodb(scientist, Scientist, from_institution_api=True)


def pobierz_ludzi_z_uczelni(client: PBNClient, instutition_id):
    """
    Ta procedura pobierze dane wszystkich osob z uczelni, mozna uruchamiac
    zamiast pobierz_ludzi
    """
    assert instutition_id is not None

    Scientist.objects.filter(from_institution_api=True).update(
        from_institution_api=False
    )

    pool = initialize_pool()
    results = []
    elementy = client.get_people_by_institution_id(instutition_id)

    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()

    for person in pbar(elementy, count=len(elementy), label="pobierz ludzi z uczelni"):
        result = pool.apply_async(
            _single_unit_ludzie_z_uczelni, args=(client, person["personId"])
        )
        results.append(result)
    wait_for_results(pool, results)


def integruj_autorow_z_uczelni(client: PBNClient, instutition_id):
    """
    Ta procedure uruchamiamy dopiero po zaciągnięciu bazy osób.
    """
    for person in Scientist.objects.filter(from_institution_api=True):
        pbn_id = None
        if person.value("object", "legacyIdentifiers", return_none=True):
            pbn_id = person.value("object", "legacyIdentifiers")[0]

        autor = matchuj_autora(
            imiona=person.value("object", "name", return_none=True),
            nazwisko=person.value("object", "lastName", return_none=True),
            orcid=person.value("object", "orcid", return_none=True),
            pbn_id=pbn_id,
            pbn_uid_id=person.pk,
            tytul_str=person.value("object", "qualifications", return_none=True),
        )
        if autor is None:
            print(f"Brak dopasowania w jednostce dla autora {person}")
            continue

        if autor.pbn_uid_id is None:
            autor.pbn_uid_id = person.mongoId
            autor.save()

        if autor.pbn_uid_id != person.pk:
            print(
                f"UWAGA Zmieniam powiązanie PBN UID dla autora {autor} z {autor.pbn_uid_id} na {person.pk}"
            )
            autor.pbn_uid = person
            autor.save()


def weryfikuj_orcidy(client: PBNClient, instutition_id):
    # Sprawdź dla każdego autora który ma ORCID ale nie ma PBN UID
    # czy ten ORCID występuje w bazie PBNu, odpowiednio ustawiając
    # flagę rekordu Autor:

    qry = Autor.objects.exclude(orcid=None).filter(pbn_uid_id=None)

    for autor in pbar(qry):
        res = client.get_person_by_orcid(autor.orcid)
        if not res:
            autor.orcid_w_pbn = False
            autor.save()
            continue

        sciencist = zapisz_mongodb(res[0], Scientist)
        autor.pbn_uid = sciencist
        autor.save()
        print(
            f"Dla autora {autor} utworzono powiazanie z rekordem PBN {sciencist} po ORCID"
        )


def matchuj_autora_po_stronie_pbn(imiona, nazwisko, orcid):
    if orcid is not None:

        # Szukamy w rekordach zaimportowanych przez API instytucji

        qry = Q(versions__contains=[{"current": True, "object": {"orcid": orcid}}]) & Q(
            from_institution_api=True
        )
        try:
            res = Scientist.objects.get(qry)
            return res
        except Scientist.DoesNotExist:
            pass
        except Scientist.MultipleObjectsReturned:
            print(
                f"XXX ORCID istnieje wiele razy w bazie PBN w rekordach importowanych przez API instytucji {orcid}"
            )
            for elem in Scientist.objects.filter(qry):
                print(
                    "\t * ",
                    elem.pk,
                    elem.name,
                    elem.lastName,
                )

        # Szukamy w rekordach wszystkich przez API instytucji

        qry = Q(versions__contains=[{"current": True, "object": {"orcid": orcid}}])
        try:
            res = Scientist.objects.exclude(from_institution_api=True).get(qry)
            return res
        except Scientist.DoesNotExist:
            print(
                f"*** ORCID nie istnieje w rekordach ani z API instytucji, ani we wszystkich {orcid}"
            )
        except Scientist.MultipleObjectsReturned:
            print(
                f"XXX ORCID istnieje wiele razy w bazie PBN w rekordach importowanych nie-przez API instytucji {orcid}"
            )
            for elem in Scientist.objects.filter(qry):
                print(
                    "\t * ",
                    elem.pk,
                    elem.name,
                    elem.lastName,
                )

    qry = Q(
        versions__contains=[
            {
                "current": True,
                "object": {"lastName": nazwisko.strip(), "name": imiona.strip()},
            }
        ]
    )
    try:
        res = Scientist.objects.filter(from_institution_api=True).get(qry)
        return res
    except Scientist.DoesNotExist:
        print(
            f"*** BRAK AUTORA w PBN z API instytucji, istnieje w BPP (im/naz): {nazwisko} {imiona}"
        )
    except Scientist.MultipleObjectsReturned:
        print(
            f"XXX AUTOR istnieje wiele razy w bazie PBN z API INSTYTUCJI (im/naz) {nazwisko} {imiona}"
        )

    # Autorzy nie-z-API instytucji

    try:
        res = Scientist.objects.exclude(from_institution_api=True).get(qry)
        return res
    except Scientist.DoesNotExist:
        print(
            f"*** BRAK AUTORA w PBN z danych spoza API instytucji, istnieje w BPP: {nazwisko} {imiona}"
        )
    except Scientist.MultipleObjectsReturned:
        print(
            f"XXX AUTOR istnieje wiele razy w bazie PBN z danych "
            f"spoza API INSTYTUCJI {nazwisko} {imiona}, "
            f"próba dobrania najlepszego"
        )

        can_be_set = False
        rated_elems = []
        for elem in Scientist.objects.exclude(from_institution_api=True).filter(qry):
            cur_elem_points = 0
            for attr in [
                "currentEmployments",
                "externalIdentifiers",
                "legacyIdentifiers",
                "qualifications",
            ]:
                if elem.value_or_none("object", attr):
                    cur_elem_points += 1

            currentEmployments = elem.value_or_none("object", "currentEmployments")
            if currentEmployments is not None:
                for pos in currentEmployments:
                    if pos.get("institutionId") == Uczelnia.objects.default.pbn_uid_id:
                        can_be_set = True

            rated_elems.append((cur_elem_points, elem.pk))

        rated_elems.sort(reverse=True)
        if can_be_set:
            print(f"--> Sposrod elementow {rated_elems} wybieram pierwszy")
            return Scientist.objects.get(pk=rated_elems[0][1])
        else:
            print(
                f"XXX Sposrod elementow {rated_elems} NIE WYBIERAM NIC, bo autor nie pracuje w jednostce"
            )


def integruj_wszystkich_niezintegrowanych_autorow():
    autorzy_z_dyscyplina_ids = Autor_Dyscyplina.objects.values("autor_id").distinct()

    for autor in Autor.objects.filter(pk__in=autorzy_z_dyscyplina_ids, pbn_uid_id=None):
        sciencist = matchuj_autora_po_stronie_pbn(
            autor.imiona, autor.nazwisko, autor.orcid
        )
        if sciencist:
            print(f"==> integracja wszystkich: ustawiam {autor} na {sciencist.pk}")
            autor.pbn_uid = sciencist
            autor.save()


def integruj_zrodla(disable_progress_bar):
    def fun(qry):
        found = False
        try:
            u = Journal.objects.get(qry)
            found = True
        except Journal.DoesNotExist:
            return False
        except Journal.MultipleObjectsReturned:
            # warnings.warn(
            #     f"Znaleziono liczne dopasowania w PBN dla {zrodlo}, szukam czy jakies ma mniswId"
            # )
            for u in Journal.objects.filter(qry):
                if u.mniswId is not None:
                    found = True
                    break

            if not found:
                print(
                    f"Znaleziono liczne dopasowania w PBN dla {zrodlo}, żadnie nie ma mniswId, wybieram to z "
                    f"najdłuższym opisem"
                )
                u = (
                    Journal.objects.filter(qry)
                    .annotate(
                        json_len=Func(
                            F("versions"),
                            function="pg_column_size",
                            output_field=IntegerField(),
                        )
                    )
                    .order_by("-json_len")
                    .first()
                )

        zrodlo.pbn_uid = u
        zrodlo.save()
        return True

    for zrodlo in pbar(
        Zrodlo.objects.filter(pbn_uid_id=None),
        label="Integracja zrodel",
        disable_progress_bar=disable_progress_bar,
    ):
        queries = []

        if zrodlo.issn:
            queries.append(Q(issn=zrodlo.issn) | Q(eissn=zrodlo.issn))

        if zrodlo.e_issn:
            queries.append(Q(eissn=zrodlo.e_issn) | Q(issn=zrodlo.e_issn))

        queries.append(Q(title__iexact=zrodlo.nazwa))

        if fun(reduce(operator.or_, queries)):
            print(f"\nZNALEZIONO dopasowania w PBN dla {zrodlo} -> {zrodlo.pbn_uid}")
            continue

        print(f"\nNie znaleziono dopasowania w PBN dla {zrodlo}")


@transaction.atomic
def integruj_wydawcow():
    # Najpierw dopasuj wydawcow z MNISWId
    for elem in pbar(
        Publisher.objects.filter(versions__contains=[{"current": True}]).filter(
            versions__0__object__has_key="mniswId"
        ),
        label="match wyd mnisw",
    ):
        w = matchuj_wydawce(elem.value("object", "publisherName"))
        if w is not None:
            if w.pbn_uid_id is None and w.pbn_uid_id != elem.pk:
                print(f"Przypisuje Wydawca PBN {elem} -> wydawca BPP {w}")
                w.pbn_uid_id = elem.pk
                w.save()

    # Dopasuj wszystkich
    for elem in pbar(
        Publisher.objects.filter(versions__contains=[{"current": True}]),
        label="match wyd wszyscy",
    ):
        w = matchuj_wydawce(elem.value("object", "publisherName"))
        if w is not None:
            if w.pbn_uid_id is None and w.pbn_uid_id != elem.pk:
                print(f"Przypisuje Wydawca PBN {elem} -> wydawca BPP {w}")
                w.pbn_uid_id = elem.pk
                w.save()


def zweryfikuj_lub_stworz_match(elem, bpp_rekord):
    if bpp_rekord is not None:
        if bpp_rekord.pbn_uid_id is not None and bpp_rekord.pbn_uid_id != elem.pk:
            print(
                f"\r\n*** Rekord BPP {bpp_rekord} ma już PBN UID {bpp_rekord.pbn_uid_id}, "
                f"pasuje też do {elem} PBN UID {elem.pk}"
            )
            return

        if bpp_rekord.pbn_uid_id is None:
            p = bpp_rekord.original
            p.pbn_uid_id = elem.pk
            p.save(update_fields=["pbn_uid_id"])


def _integruj_single_part(ids):

    for _id in ids:
        try:
            elem = Publication.objects.get(pk=_id)
        except Publication.DoesNotExist as e:
            print(f"Brak publikacji o ID {_id}")
            raise e
        p = elem.matchuj_do_rekordu_bpp()
        zweryfikuj_lub_stworz_match(elem, p)


def split_list(lst, n):
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


CPU_COUNT = "auto"

DEFAULT_CONTEXT = "spawn"


def _init():
    import django

    django.setup()


def initialize_pool(multipler=1):
    global CPU_COUNT

    if CPU_COUNT == "auto":
        cpu_count = os.cpu_count() * 3 // 4
        if cpu_count < 1:
            cpu_count = 1

        cpu_count = cpu_count * multipler
        if cpu_count < 1:
            cpu_count = 1

    elif CPU_COUNT == "single":
        cpu_count = 1
    else:
        raise NotImplementedError(f"CPU_COUNT = {CPU_COUNT}")

    return multiprocessing.get_context(DEFAULT_CONTEXT).Pool(
        cpu_count, initializer=_init
    )


def _integruj_publikacje(
    pubs, disable_multiprocessing=False, skip_pages=0, label="_integruj_publikacje"
):

    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()
    pool = initialize_pool()

    BATCH_SIZE = 128
    results = []
    for no, elem in enumerate(split_list(pubs, BATCH_SIZE)):
        if no < skip_pages:
            continue

        if disable_multiprocessing:
            _integruj_single_part(elem)
            print(f"{label} {no} of {len(pubs)//BATCH_SIZE}...", end="\r")
            sys.stdout.flush()
        else:
            result = pool.apply_async(_integruj_single_part, args=(elem,))
            results.append(result)

    wait_for_results(pool, results, label=label)


def integruj_wszystkie_publikacje(
    disable_multiprocessing=False, ignore_already_matched=False, skip_pages=0
):
    pubs = Publication.objects.all()

    if ignore_already_matched:
        pubs = pubs.exclude(
            pk__in=Rekord.objects.exclude(pbn_uid_id=None)
            .values_list("pbn_uid_id", flat=True)
            .distinct()
        )

    pubs = pubs.order_by("-pk")
    pubs = list(pubs.values_list("pk", flat=True).distinct())

    return _integruj_publikacje(
        pubs, disable_multiprocessing=disable_multiprocessing, skip_pages=skip_pages
    )


def integruj_publikacje_instytucji(
    disable_multiprocessing=False, ignore_already_matched=False, skip_pages=0
):
    """
    :param ignore_already_matched: jeżeli True, to publikacje, które już mają swój match
    po stronie BPP nie będa analizowane.

    """

    pubs = (
        OswiadczenieInstytucji.objects.all()
        .values_list("publicationId_id", flat=True)
        .order_by("-pk")
        .distinct()
    )
    return _integruj_publikacje(
        pubs, disable_multiprocessing=disable_multiprocessing, skip_pages=skip_pages
    )


MODELE_Z_PBN_UID = (
    Wydawnictwo_Zwarte,
    Wydawnictwo_Ciagle,
    Praca_Doktorska,
    Praca_Habilitacyjna,
)


#
# Poniżej wersja iterująca po rekordach w BPP, szukająca matchy w pbn_api.Publication
#
# def integruj_publikacje(
#     disable_multiprocessing=False, ignore_already_matched=False, skip_pages=0
# ):
#     """
#     :param ignore_already_matched: jeżeli True, to publikacje, które już mają swój match
#     po stronie BPP nie będa analizowane.
#
#     """
#     for klass in MODELE_Z_PBN_UID:
#         for elem in pbar(klass.objects.all()):
#             zrodlo = None
#             if hasattr(elem, "zrodlo"):
#                 zrodlo = elem.zrodlo
#
#             isbn = None
#             if hasattr(elem, "isbn"):
#                 isbn = elem.isbn
#
#             res = matchuj_pbn_api_publication(
#                 elem.tytul_oryginalny,
#                 elem.rok,
#                 elem.doi,
#                 elem.public_www or elem.www,
#                 isbn,
#                 zrodlo,
#             )
#
#             if res is not None:
#                 if elem.pbn_uid_id is not None:
#                     if elem.pbn_uid_id != res.pk:
#                         print(
#                             f"XXX Publikacja {elem} ma juz PBN UID {elem.pbn_uid_id} {elem.pbn_uid} ale "
#                             f"matchowanie chce ją przypisać do {res.pk} {res}"
#                         )
#                 else:
#                     # elem.pbn_uid_id is None
#                     elem.pbn_uid_id = res.pk
#                     elem.save(update_fields=["pbn_uid_id"])
#             else:
#                 # print(f"XXX Brak matchu w PBN: {elem}")
#                 pass
#

PBN_KOMUNIKAT_ISBN_ISTNIEJE = "Publikacja o identycznym ISBN lub ISMN już istnieje"
PBN_KOMUNIKAT_DOI_ISTNIEJE = "Publikacja o identycznym DOI i typie już istnieje"


def _synchronizuj_pojedyncza_publikacje(
    client,
    rec,
    force_upload=False,
    export_pk_zero=None,
    delete_statements_before_upload=False,
):
    try:
        client.sync_publication(
            rec,
            force_upload=force_upload,
            export_pk_zero=export_pk_zero,
            delete_statements_before_upload=delete_statements_before_upload,
        )
    except SameDataUploadedRecently:
        pass
    except HttpException as e:
        if e.status_code == 400:
            if rec.pbn_uid_id is not None and (
                PBN_KOMUNIKAT_ISBN_ISTNIEJE in e.content
                or PBN_KOMUNIKAT_DOI_ISTNIEJE in e.content
            ):

                warnings.warn(
                    f"UWAGA: rekord z BPP {rec} mimo posiadania PBN UID {rec.pbn_uid_id} dostał"
                    f"przy synchronizacji komunkat: {e.content} !! Sprawa DO SPRAWDZENIA RECZNIE"
                )
                return

            ret = None

            if PBN_KOMUNIKAT_ISBN_ISTNIEJE in e.content:
                if (
                    hasattr(rec, "isbn")
                    and hasattr(rec, "e_isbn")
                    and (rec.isbn is not None or rec.e_isbn is not None)
                    and (normalize_isbn(rec.isbn) or normalize_isbn(rec.e_isbn))
                ):
                    isbn_value = normalize_isbn(rec.isbn or rec.e_isbn)
                    ret = _pobierz_prace_po_elemencie(
                        client, "isbn", isbn_value, value_from_rec=rec
                    )

            elif PBN_KOMUNIKAT_DOI_ISTNIEJE in e.content:
                if (
                    hasattr(rec, "doi")
                    and rec.doi is not None
                    and normalize_doi(rec.doi)
                ):
                    doi_value = normalize_doi(rec.doi)
                    ret = _pobierz_prace_po_elemencie(
                        client, "doi", doi_value, value_from_rec=rec
                    )

            else:
                warnings.warn(
                    f"{rec.pk},{rec.tytul_oryginalny},{rec.rok},PBN Server Error: {e.content}"
                )
                return

            if ret is None or rec not in ret:
                # Jeżeli rekordu synchronizowanego nie ma wśród rekordów pobranych - zmatchowanych
                # przez funkcję _pobierz_prace_po_elemencie, to wyjdź teraz:
                return

            # Rekord znalazł się na liście zmatchowanych przez funkcję _pobierz_po_elemencie,
            # nadano mu pbn_uid_id. Odśwież to ID z bazy, uruchom jeszcze raz synchronizację
            # rekordu:
            rec.refresh_from_db()
            assert rec.pbn_uid_id is not None
            return _synchronizuj_pojedyncza_publikacje(client, rec)

        if e.status_code == 500:
            warnings.warn(
                f"{rec.pk},{rec.tytul_oryginalny},{rec.rok},PBN Server Error: {e.content}"
            )

        if e.status_code == 403:
            # Jezeli przytrafi się wyjątek taki, jak poniżej:
            #
            # pbn_api.exceptions.HttpException: (403, '/api/v1/publications', '{"code":403,"message":"Forbidden",
            # "description":"W celu poprawnej autentykacji należy podać poprawny token użytkownika aplikacji.
            # Podany token użytkownika X w ramach aplikacji Y nie istnieje lub został unieważniony!"}')
            #
            # to go podnieś. Jeżeli autoryzacja nie została przeprowadzona poprawnie, to nie chcemy kontynuować
            # dalszego procesu synchronizacji prac.
            raise e

    except WillNotExportError as e:
        warnings.warn(
            f"{rec.pk},{rec.tytul_oryginalny},{rec.rok},nie wyeksportuje, bo: {e}"
        )


def wydawnictwa_zwarte_do_synchronizacji():
    return (
        Wydawnictwo_Zwarte.objects.filter(rok__gte=PBN_MIN_ROK)
        .exclude(charakter_formalny__rodzaj_pbn=None)
        .exclude(isbn=None, e_isbn=None)
        .exclude(jezyk__pbn_uid_id=None)
        .exclude(
            doi=None,
            public_www=None,
            www=None,
            wydawnictwo_nadrzedne__www=None,
            wydawnictwo_nadrzedne__public_www=None,
        )
        # rekordy bez WWW wysylamy gdy jest okreslone nadrzedne + nadrzende ma WWW
    )


def wydawnictwa_ciagle_do_synchronizacji():
    return (
        Wydawnictwo_Ciagle.objects.filter(rok__gte=PBN_MIN_ROK)
        .exclude(jezyk__pbn_uid_id=None)
        .exclude(charakter_formalny__rodzaj_pbn=None)
        .exclude(doi=None, public_www=None, www=None)
    )


def synchronizuj_publikacje(
    client,
    force_upload=False,
    only_bad=False,
    only_new=False,
    skip=0,
    export_pk_zero=None,
    delete_statements_before_upload=False,
):
    """
    Komenda eksportująca wszystkie publikacje w systmeie, zaczynając od książek,
    przez rozdziały, po wydawnictwa ciągłe.


    :param only_bad: eksportuj jedynie rekordy, które mają wpis w tabeli SentData, ze ich eksport
    nie powiódł się wcześniej.

    :param only_new: eksportuj jedynie rekordy, które nie mają wpisu w tabeli SentData,

    :param force_upload: wymuszaj ponowne wysłanie, niezależnie od sytuacji w tabeli SentData
    """

    assert not (only_bad and only_new), "Te parametry wykluczają się wzajemnie"
    #
    # Wydawnictwa zwarte
    #
    zwarte_baza = wydawnictwa_zwarte_do_synchronizacji()

    if only_bad:
        zwarte_baza = zwarte_baza.filter(
            pk__in=SentData.objects.bad_uploads(Wydawnictwo_Zwarte)
        )

    if only_new:
        # Nie synchronizuj prac ktore juz sa w SentData
        zwarte_baza = zwarte_baza.exclude(
            pk__in=SentData.objects.ids_for_model(Wydawnictwo_Zwarte)
            .values_list("pk", flat=True)
            .distinct()
        )

    for rec in pbar(
        zwarte_baza.filter(wydawnictwo_nadrzedne_id=None),
        label="sync_zwarte_ksiazki",
    ):
        _synchronizuj_pojedyncza_publikacje(
            client,
            rec,
            force_upload=force_upload,
            delete_statements_before_upload=delete_statements_before_upload,
            export_pk_zero=export_pk_zero,
        )

    for rec in pbar(
        zwarte_baza.exclude(wydawnictwo_nadrzedne_id=None),
        label="sync_zwarte_rozdzialy",
    ):
        _synchronizuj_pojedyncza_publikacje(
            client,
            rec,
            force_upload=force_upload,
            delete_statements_before_upload=delete_statements_before_upload,
            export_pk_zero=export_pk_zero,
        )

    #
    # Wydawnicwa ciagle
    #
    ciagle_baza = wydawnictwa_ciagle_do_synchronizacji()

    if only_bad:
        ciagle_baza = ciagle_baza.filter(
            pk__in=SentData.objects.bad_uploads(Wydawnictwo_Ciagle)
        )

    if only_new:
        # Nie synchronizuj prac ktore juz sa w SentData
        ciagle_baza = ciagle_baza.exclude(
            pk__in=SentData.objects.ids_for_model(Wydawnictwo_Ciagle)
            .values_list("pk", flat=True)
            .distinct()
        )

    for rec in pbar(
        ciagle_baza,
        label="sync_ciagle",
    ):
        _synchronizuj_pojedyncza_publikacje(
            client,
            rec,
            force_upload=force_upload,
            delete_statements_before_upload=delete_statements_before_upload,
            export_pk_zero=export_pk_zero,
        )


def clear_match_publications():
    for model in MODELE_Z_PBN_UID:
        print(f"Setting pbn_uid_ids of {model} to null...")
        model.objects.exclude(pbn_uid_id=None).update(pbn_uid_id=None)


def clear_publications():
    clear_match_publications()
    for model in [OswiadczenieInstytucji, PublikacjaInstytucji, Publication, SentData]:
        model.objects.all()._raw_delete(MODELE_Z_PBN_UID[0].objects.db)


def clear_all():
    for model in (
        Autor,
        Jednostka,
        Wydawca,
        Jezyk,
        Uczelnia,
        Zrodlo,
    ):
        print(f"Setting pbn_uid_ids of {model} to null...")
        model.objects.exclude(pbn_uid_id=None).update(pbn_uid_id=None)

    clear_publications()

    for model in (
        Language,
        Country,
        Institution,
        Conference,
        Journal,
        Publisher,
        Scientist,
    ):
        print(f"Deleting all {model}")
        model.objects.all()._raw_delete(model.objects.db)


def integruj_oswiadczenia_z_instytucji():
    noted_pub = set()
    noted_aut = set()
    for elem in pbar(
        OswiadczenieInstytucji.objects.filter(inOrcid=True),
        label="integruj_oswiadczenia_z_instytucji",
    ):
        pub = elem.get_bpp_publication()
        if pub is None:
            pub = elem.publicationId.matchuj_do_rekordu_bpp()

            if pub is None:
                if elem.publicationId_id not in noted_pub:
                    print(
                        f"\r\nPPP Brak odpowiednika publikacji w BPP dla pracy {elem.publicationId}, "
                        f"parametr inOrcid dla tej pracy nie zostanie zaimportowany!"
                    )
                    noted_pub.add(elem.publicationId_id)
                continue
            else:
                zweryfikuj_lub_stworz_match(elem.publicationId, pub)

        aut = elem.get_bpp_autor()
        if aut is None:
            if elem.personId_id not in noted_aut:
                print(
                    f"\r\nAAA Brak odpowiednika autora w BPP dla autora {elem.personId}, "
                    f"parametr inOrcid dla tego autora nie zostanie zaimportowany!"
                )
                noted_aut.add(elem.publicationId_id)
            continue

        try:
            rec = pub.autorzy_set.get(autor=aut)
        except pub.autorzy_set.model.DoesNotExist:
            print(
                f"XXX Po stronie PBN: {elem.publicationId}, \n"
                f"XXX po stronie BPP: {pub}, {aut} -- nie ma ta praca takiego autora!"
            )

        if not rec.profil_orcid:
            rec.profil_orcid = True
            rec.save(update_fields=["profil_orcid"])


def wyswietl_niezmatchowane_ze_zblizonymi_tytulami():
    for model, klass in [
        (Wydawnictwo_Zwarte, Wydawnictwo_Zwarte_Autor),
        (Wydawnictwo_Ciagle, Wydawnictwo_Ciagle_Autor),
    ]:
        for rekord in (
            model.objects.filter(
                pk__in=klass.objects.exclude(dyscyplina_naukowa=None)
                .filter(rekord__pbn_uid_id=None)
                .values("rekord")
                .distinct()
            )
            .annotate(tytul_oryginalny_length=Length("tytul_oryginalny"))
            .filter(tytul_oryginalny_length__gte=10)
        ):
            print(
                f"\r\nRekord z dyscyplinami, bez dopasowania w PBN: {rekord.rok} {rekord}"
            )

            nt = normalize_tytul_publikacji(rekord.tytul_oryginalny)

            res = (
                Publication.objects.annotate(
                    similarity=TrigramSimilarity("title", nt),
                )
                .filter(year=rekord.rok)
                .filter(similarity__gt=0.5)
                .order_by("-similarity")
            )

            for elem in res[:5]:
                print("- MOZE: ", elem.mongoId, elem.title, elem.similarity)


def sprawdz_ilosc_autorow_przy_zmatchowaniu():
    res = csv.writer(open("rozne-ilosci-autorow.csv", "w"))
    res.writerow(
        [
            "Komunikat",
            "Praca w BPP",
            "Praca w PBN",
            "Rok",
            "Autorzy w BPP",
            "Autorzy w PBN",
            "Nazwiska w BPP",
            "Nazwiska w PBN",
        ]
    )
    for praca in Rekord.objects.exclude(pbn_uid_id=None).order_by(
        "rok", "tytul_oryginalny"
    ):
        ca = praca.autorzy_set.all().count()
        pa = praca.pbn_uid.policz_autorow()
        if ca != pa:
            res.writerow(
                [
                    "Rozna ilosc autorow",
                    str(praca),
                    str(praca.pbn_uid),
                    str(praca.rok),
                    str(ca),
                    str(pa),
                    str(praca.opis_bibliograficzny_zapisani_autorzy_cache),
                    str(praca.pbn_uid.autorzy),
                ]
            )


def _pobierz_pojedyncza_prace(client, publicationId):
    try:
        data = client.get_publication_by_id(publicationId)
    except HttpException as e:
        if e.status_code == 500 and "Internal server error" in e.content:
            print(
                f"\r\nSerwer PBN zwrocil blad 500 dla PBN UID {publicationId} --> {e.content}"
            )
            return
        raise e
    zapisz_mongodb(data, Publication, client)


def pobierz_rekordy_publikacji_instytucji(client: PBNClient):
    seen = set()
    for elem in pbar(
        client.get_institution_publications(page_size=1000),
        label="scan pobierz_rekordy_publikacji_instytucji",
    ):
        publicationId = elem["publicationId"]
        seen.add(publicationId)

    _bede_uzywal_bazy_danych_z_multiprocessing_z_django()
    pool = initialize_pool(multipler=2)

    args = [(client, _id) for _id in seen]
    for _ in pbar(
        pool.istarmap(_pobierz_pojedyncza_prace, args),
        label="pobierz_rekordy_publikacj_instytucji",
        count=len(args),
    ):
        pass

    pool.join()
    pool.close()


def usun_wszystkie_oswiadczenia(client):
    for elem in pbar(
        OswiadczenieInstytucji.objects.all(), label="usun_wszystkie_oswiadczenia"
    ):
        with transaction.atomic():
            try:
                elem.sprobuj_skasowac_z_pbn(pbn_client=client)
            except StatementDeletionError as e:
                if e.status_code == 400 and (
                    "Nie istnieją oświadczenia" in e.content
                    or "Nie istnieje oświadczenie" in e.content
                ):
                    pass
                else:
                    raise e

            elem.delete()


def usun_zerowe_oswiadczenia(client):

    for klass in Wydawnictwo_Ciagle, Wydawnictwo_Zwarte:
        zerowe = klass.objects.exclude(pbn_uid=None).filter(punkty_kbn=0)

        for elem in pbar(
            OswiadczenieInstytucji.objects.filter(
                publicationId_id__in=zerowe.values_list("pbn_uid_id", flat=True)
            ),
            label=f"usun_zerowe dla {klass}",
        ):
            with transaction.atomic():
                try:
                    elem.sprobuj_skasowac_z_pbn(pbn_client=client)
                except StatementDeletionError as e:
                    if e.status_code == 400 and (
                        "Nie istnieją oświadczenia" in e.content
                        or "Nie istnieje oświadczenie" in e.content
                    ):
                        pass
                    else:
                        raise e

                elem.delete()



# Generated by Django 3.0.11 on 2021-04-25 23:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0260_jednostka_pbn_uid"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_api_root",
            field=models.URLField(
                default="https://pbn-micro-alpha.opi.org.pl",
                verbose_name="Adres API w PBN",
            ),
        ),
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_app_name",
            field=models.CharField(
                blank=True,
                max_length=128,
                null=True,
                verbose_name="Nazwa aplikacji w PBN",
            ),
        ),
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_app_token",
            field=models.CharField(
                blank=True,
                max_length=128,
                null=True,
                verbose_name="Token aplikacji w PBN",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-
from braces.views import LoginRequiredMixin
from django.http.response import JsonResponse
from django.views.generic.base import View

from bpp.models.abstract import wez_zakres_stron, parse_informacje


class StronaTomNrZeszytuView(LoginRequiredMixin, View):
    def post(self, request, *args, **kw):
        szczegoly = request.POST.get('s', '').strip()
        informacje = request.POST.get('i', '').strip()

        tom = parse_informacje(informacje, "tom")
        nr_zeszytu = parse_informacje(informacje, "numer")

        return JsonResponse({
            'strony': wez_zakres_stron(szczegoly),
            'tom': tom,
            'nr_zeszytu': nr_zeszytu

        })


# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-20 00:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0097_nagrody'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='praca_wybitna',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='uzasadnienie_wybitnosci',
            field=models.TextField(blank=True, default='', verbose_name='Uzasadnienie wybitności'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='praca_wybitna',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='uzasadnienie_wybitnosci',
            field=models.TextField(blank=True, default='', verbose_name='Uzasadnienie wybitności'),
        ),
    ]



"""
This file was generated with the custommenu management command, it contains
the classes for the admin menu, you can customize this class as you want.

To activate your custom menu add the following to your settings.py::
    ADMIN_TOOLS_MENU = 'django_bpp.menu.CustomMenu'
"""
from admin_tools.menu import Menu, items
from django.apps import apps
from django.db import connection
from django.urls import reverse

from django.contrib import admin

from django.utils.translation import ugettext_lazy as _

from bpp.const import GR_WPROWADZANIE_DANYCH, GR_ZGLOSZENIA_PUBLIKACJI

PBN_MENU = [
    ("Instytucje", "/admin/pbn_api/institution"),
    ("Konferencje", "/admin/pbn_api/conference"),
    ("Zrodla", "/admin/pbn_api/journal"),
    ("Wydawcy", "/admin/pbn_api/publisher"),
    ("Naukowcy", "/admin/pbn_api/scientist"),
    ("Publikacje", "/admin/pbn_api/publication"),
    # Chowamy publikacjie instytucji z menu - niech sobie zostaną dostępne z innego
    # miejsca, ale nie są one generalnie potrzebne (mpasternak, 1.09.2021)
    # ("Publikacje instytucji", "/admin/pbn_api/publikacjainstytucji"),
    ("Oświadczenia instytucji", "/admin/pbn_api/oswiadczenieinstytucji"),
    ("Przesłane dane", "/admin/pbn_api/sentdata"),
]

IMPORT_DBF_MENU_1 = [
    (
        "Zaimportowana bibliografia",
        "/admin/import_dbf/bib/",
    ),
    (
        "Zaimportowane dane Open Access",
        "/admin/import_dbf/b_u/",
    ),
    (
        "Zaimportowane opisy rekordów",
        "/admin/import_dbf/poz/",
    ),
    (
        "Zaimportowane i zanalizowane opisy rekordów",
        "/admin/import_dbf/bib_desc/",
    ),
    (
        "Zaimportowane źródła",
        "/admin/import_dbf/usi/",
    ),
    (
        "Zaimportowane jednostki",
        "/admin/import_dbf/jed/",
    ),
    (
        "Zaimportowani autorzy",
        "/admin/import_dbf/aut/",
    ),
    (
        "Zaimportowane powiązania autor-rekord",
        "/admin/import_dbf/b_a/",
    ),
    # Tabela sesji - zbędna
    # ('Zaimportowane Ses', '/admin/import_dbf/ses/',),
    (
        "Zaimportowane identyfikatory PBN",
        "/admin/import_dbf/ixn/",
    ),
    (
        "Zaimportowane dyscypliny pracowników",
        "/admin/import_dbf/dys/",
    ),
    (
        "Zaimportowane hasła naukowe",
        "/admin/import_dbf/ixe/",
    ),
    (
        "Zaimportowane charaktery publikacji",
        "/admin/import_dbf/pub/",
    ),
    (
        "Zaimportowana wersja systemu",
        "/admin/import_dbf/sys/",
    ),
    (
        "Zaimportowane wydziały",
        "/admin/import_dbf/wyd/",
    ),
    (
        "Zaimportowane dziedziny",
        "/admin/import_dbf/ldy/",
    ),
    (
        "Zaimportowane języki",
        "/admin/import_dbf/jez/",
    ),
    (
        "Zaimportowane typy KBN",
        "/admin/import_dbf/kbn/",
    ),
    (
        "Zaimportowane bazy",
        "/admin/import_dbf/ixb/",
    ),
    (
        "Zaimportowane listy wydawców",
        "/admin/import_dbf/lis/",
    ),
    (
        "Zaimportowane historia jednostek",
        "/admin/import_dbf/j_h/",
    ),
    (
        "Zaimportowane rekordy KBR",
        "/admin/import_dbf/kbr/",
    ),
    (
        "Zaimportowane Jer",
        "/admin/import_dbf/jer/",
    ),
    (
        "Zaimportowane B_B",
        "/admin/import_dbf/b_b/",
    ),
    (
        "Zaimportowane B_N",
        "/admin/import_dbf/b_n/",
    ),
]

IMPORT_DBF_MENU_2 = [
    (
        "Zaimportowane Wx2",
        "/admin/import_dbf/wx2/",
    ),
    (
        "Zaimportowane Kad",
        "/admin/import_dbf/kad/",
    ),
    (
        "Zaimportowane Loc",
        "/admin/import_dbf/loc/",
    ),
    (
        "Zaimportowane Pbc",
        "/admin/import_dbf/pbc/",
    ),
    (
        "Zaimportowane Sci",
        "/admin/import_dbf/sci/",
    ),
    (
        "Zaimportowane Wsx",
        "/admin/import_dbf/wsx/",
    ),
    (
        "Zaimportowane B_E",
        "/admin/import_dbf/b_e/",
    ),
    (
        "Zaimportowane B_P",
        "/admin/import_dbf/b_p/",
    ),
    (
        "Zaimportowane Ixp",
        "/admin/import_dbf/ixp/",
    ),
    (
        "Zaimportowane Pba",
        "/admin/import_dbf/pba/",
    ),
    (
        "Zaimportowane Pbd",
        "/admin/import_dbf/pbd/",
    ),
    (
        "Zaimportowane Rtf",
        "/admin/import_dbf/rtf/",
    ),
    (
        "Zaimportowane S_B",
        "/admin/import_dbf/s_b/",
    ),
    (
        "Zaimportowane Wsy",
        "/admin/import_dbf/wsy/",
    ),
    (
        "Zaimportowane B_L",
        "/admin/import_dbf/b_l/",
    ),
    (
        "Zaimportowane Ext",
        "/admin/import_dbf/ext/",
    ),
    (
        "Zaimportowane Pbb",
        "/admin/import_dbf/pbb/",
    ),
]

SYSTEM_MENU = [
    ("Charaktery formalne", "/admin/bpp/charakter_formalny/"),
    ("Charakter PBN", "/admin/bpp/charakter_pbn/"),
    ("Dyscypliny naukowe", "/admin/bpp/dyscyplina_naukowa/"),
    ("Formularze - wartości domyślne", "/admin/formdefaults/formrepresentation/"),
    ("Funkcje w jednostce", "/admin/bpp/funkcja_autora/"),
    ("Granty", "/admin/bpp/grant/"),
    ("Grupy pracownicze", "/admin/bpp/grupa_pracownicza/"),
    ("Języki", "/admin/bpp/jezyk/"),
    ("OpenAccess: wydawnictwa ciągłe", "/admin/bpp/tryb_openaccess_wydawnictwo_ciagle"),
    ("OpenAccess: wydawnictwa zwarte", "/admin/bpp/tryb_openaccess_wydawnictwo_zwarte"),
    ("OpenAccess: czas udostępnienia", "/admin/bpp/czas_udostepnienia_openaccess"),
    ("OpenAccess: licencja", "/admin/bpp/licencja_openaccess"),
    ("OpenAccess: wersja tekstu", "/admin/bpp/wersja_tekstu_openaccess"),
    ("Organy przyznające nagrody", "/admin/bpp/organprzyznajacynagrody/"),
    ("Rodzaje źródeł", "/admin/bpp/rodzaj_zrodla/"),
    ("Rodzaje praw patentowych", "/admin/bpp/rodzaj_prawa_patentowego/"),
    ("Statusy korekt", "/admin/bpp/status_korekty/"),
    ("Typy KBN", "/admin/bpp/typ_kbn/"),
    ("Typy odpowiedzialności", "/admin/bpp/typ_odpowiedzialnosci/"),
    ("Tytuły", "/admin/bpp/tytul/"),
    ("Wydawcy", "/admin/bpp/wydawca/"),
    ("Wymiary etatów", "/admin/bpp/wymiar_etatu/"),
    ("Widoczność opcji wyszukiwarki", "/admin/bpp/bppmultiseekvisibility/"),
    ("Zewnętrzne bazy danych", "/admin/bpp/zewnetrzna_baza_danych/"),
    ("Źródło informacji", "/admin/bpp/zrodlo_informacji/"),
]

RAPORTY_MENU = [
    ("Raporty", "/admin/flexible_reports/report/"),
    ("Źródła danych", "/admin/flexible_reports/datasource/"),
    ("Tabele", "/admin/flexible_reports/table/"),
]

WEB_MENU = [
    ("robots.txt - URLe", "/admin/robots/url/"),
    ("robots.txt - reguły", "/admin/robots/rule/"),
    ("Serwisy", "/admin/sites/site/"),
    ("Miniblog", "/admin/miniblog/article/"),
    ("Favicon", "/admin/favicon/"),
    ("Szablony", "/admin/dbtemplates/template/"),
    ("Powiązania szablonów dla opisu", "/admin/bpp/szablondlaopisubibliograficznego/"),
]

STRUKTURA_MENU = [
    ("Uczelnie", "/admin/bpp/uczelnia/"),
    ("Wydziały", "/admin/bpp/wydzial/"),
    ("Jednostki", "/admin/bpp/jednostka/"),
    ("Kierunki studiów", "/admin/bpp/kierunek_studiow/"),
]

REDAKTOR_MENU = [
    ("Autorzy", "/admin/bpp/autor/"),
    (
        "Autorzy - udziały (Ewaluacja 2021)",
        "/admin/ewaluacja2021/iloscudzialowdlaautora/",
    ),
    ("Źródła", "/admin/bpp/zrodlo/"),
    ("Serie wydawnicze", "/admin/bpp/seria_wydawnicza/"),
    ("Konferencje", "/admin/bpp/konferencja/"),
    ("Wydawcy", "/admin/bpp/wydawca/"),
    ("Wydawnictwa ciągłe", "/admin/bpp/wydawnictwo_ciagle/"),
    ("Wydawnictwa zwarte", "/admin/bpp/wydawnictwo_zwarte/"),
    ("Prace doktorskie", "/admin/bpp/praca_doktorska/"),
    ("Prace habilitacyjne", "/admin/bpp/praca_habilitacyjna/"),
    ("Patenty", "/admin/bpp/patent/"),
    ("Powiązania autorów z dyscyplinami", "/admin/bpp/autor_dyscyplina/"),
    (
        "Rozbieżności dyscyplin",
        "/admin/rozbieznosci_dyscyplin/rozbieznosciview/",
    ),
    (
        "Rozbieżności dyscyplin źródeł",
        "/admin/rozbieznosci_dyscyplin/rozbieznoscizrodelview/",
    ),
    ("Zgłoszenia publikacji", "/admin/zglos_publikacje/zgloszenie_publikacji/"),
]

ADMIN_MENU = [
    ("Grupy", "/admin/auth/group/"),
    ("Użytkownicy", "/admin/bpp/bppuser/"),
    ("Formularze wyszukiwania", "/admin/multiseek/searchform/"),
    (
        "Obsługa zgłoszeń prac - wydziały",
        "/admin/zglos_publikacje/obslugujacy_zgloszenia_wydzialow/",
    ),
    (
        "Kolumny w module redagowania",
        "/admin/dynamic_columns/modeladmincolumn/",
    ),
]


def submenu(label, tuples):
    return items.MenuItem(
        label, children=[items.MenuItem(label, link) for label, link in tuples]
    )


class CustomMenu(Menu):
    def __init__(self, **kwargs):
        Menu.__init__(self, **kwargs)

        self.children += [
            items.MenuItem(_("Dashboard"), reverse("admin:index")),
            items.Bookmarks(),
        ]

    def init_with_context(self, context):
        user = context["request"].user
        if not hasattr(user, "__admin_menu_groups"):
            user.__admin_menu_groups = [x.name for x in user.cached_groups]
        groups = user.__admin_menu_groups

        def flt(n1, n2, v):
            if user.is_superuser or n1 in groups:
                self.children += [
                    submenu(n2, v),
                ]

        flt("web", "WWW", WEB_MENU)

        flt("dane systemowe", "PBN API", PBN_MENU)

        if "import_dbf_aut" in connection.introspection.table_names():
            flt("import DBF", "import DBF", IMPORT_DBF_MENU_1)
            # flt("import DBF", "import DBF #2", IMPORT_DBF_MENU_2)
        else:
            # De-register all models from other apps
            for model in apps.get_app_config("import_dbf").models.values():
                if admin.site.is_registered(model):
                    admin.site.unregister(model)

        flt("dane systemowe", "Dane systemowe", SYSTEM_MENU)
        flt("struktura", "Struktura", STRUKTURA_MENU)
        flt(GR_WPROWADZANIE_DANYCH, "Wprowadzanie danych", REDAKTOR_MENU)
        if GR_ZGLOSZENIA_PUBLIKACJI not in groups and not user.is_superuser:
            # Wyrzuć "zgłoszenia publikacji" z REDAKTOR_MENU
            del self.children[-1].children[-1]

        flt("raporty", "Raporty", RAPORTY_MENU)
        flt("administracja", "Administracja", ADMIN_MENU)

        return super().init_with_context(context)



# Generated by Django 3.2.14 on 2022-07-10 21:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("zglos_publikacje", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="content_type",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="contenttypes.contenttype",
            ),
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="email",
            field=models.EmailField(
                default="test@test.pl",
                max_length=254,
                verbose_name="E-mail zgłaszającego",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="object_id",
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="status",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "nowe zgłoszenie"),
                    (1, "dodane do bazy"),
                    (2, "wymaga uzupełnienia"),
                    (3, "odrzucono"),
                    (4, "spam"),
                ],
                default=0,
            ),
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="utworzono",
            field=models.DateTimeField(
                auto_now_add=True, null=True, verbose_name="Utworzono"
            ),
        ),
    ]



import warnings

from django.db import models
from django.db.models import JSONField

from django.utils.functional import cached_property


class BasePBNModel(models.Model):
    created_on = models.DateTimeField(auto_now_add=True)
    last_updated_on = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


MAX_TEXT_FIELD_LENGTH = 350


class BasePBNMongoDBModel(BasePBNModel):
    mongoId = models.CharField(max_length=32, primary_key=True)
    status = models.CharField(max_length=32, db_index=True)
    verificationLevel = models.CharField(max_length=32, db_index=True)
    verified = models.BooleanField(default=False, db_index=True)
    versions = JSONField()

    # Nazwy pól wyciaganych "na wierzch" do pól obiektu
    # ze słownika JSONa (pole 'values')
    pull_up_on_save = None

    def _pull_up_on_save(self):
        for attr in self.pull_up_on_save:
            if hasattr(self, f"pull_up_{attr}"):
                fn = getattr(self, f"pull_up_{attr}")
                v = fn()
            else:
                v = self.value_or_none("object", attr)

            if v is not None:
                # Tylko błędne rekordy (takie, które zawieraja pola dlugosci kilkudziesieciu kilobajtow)
                # zawieraja bardzo dlugie wpisy. Np jeden rekord w polu 'nazwisko' ma 10 kb nazwisk,
                # po przecinku. Oczywiscie, ze sa bledne. PostgreSQL jednakze ma limit na wielkosc
                # wiersza indeksu. I tego limitu bedziemy teraz przestrzegali:
                if isinstance(v, str):
                    if len(v) >= MAX_TEXT_FIELD_LENGTH:
                        v = v[:MAX_TEXT_FIELD_LENGTH]
            setattr(self, attr, v)

    def save(
        self, force_insert=False, force_update=False, using=None, update_fields=None
    ):
        if self.pull_up_on_save:
            self._pull_up_on_save()
        return super().save(
            force_insert=force_insert,
            force_update=force_update,
            using=using,
            update_fields=update_fields,
        )

    @cached_property
    def current_version(self):
        if self.versions:
            for elem in self.versions:
                if elem["current"]:
                    return elem

    def value(self, *path, return_none=False):
        v = self.current_version
        if v is None:
            warnings.warn(
                f"Model {self.__class__} with id {self.mongoId} has NO current_version!"
            )
            if return_none:
                return
            return "[brak current_version]"

        for elem in path:
            if elem in v:
                v = v[elem]
            else:
                if return_none:
                    return None
                return f"[brak {elem}]"
        return v

    def value_or_none(self, *path):
        return self.value(*path, return_none=True)

    def website(self):
        return self.value("object", "website")

    class Meta:
        abstract = True



# -*- encoding: utf-8 -*-
from crispy_forms.helper import FormHelper
from dal_select2.widgets import ListSelect2
from django import forms


def make_nav_form(url):
    class GlobalNavForm(forms.Form):
        global_nav_value = forms.CharField(
            label="",
            widget=ListSelect2(
                url=url,
                attrs={
                    "data-html": True,
                    "data-placeholder": "Wpisz, aby wyszukać...",
                    "data-minimum-input-length": 3,
                },
            ),
        )

        def __init__(self):
            super(GlobalNavForm, self).__init__()

            self.helper = FormHelper(self)
            self.helper.form_show_labels = False
            self.helper.form_tag = False

    return GlobalNavForm


GlobalNavForm = make_nav_form("bpp:navigation-autocomplete")

AdminNavForm = make_nav_form("bpp:admin-navigation-autocomplete")


def user(request):
    if request.path.startswith("/admin/"):
        return dict(global_nav_form=AdminNavForm())
    return dict(global_nav_form=GlobalNavForm())



# -*- coding: utf-8 -*-


from django.db import migrations
from django.db.migrations.operations.special import RunPython
from django.db import migrations, models

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0045_auto_20160809_0004'),
    ]

    operations = [
        migrations.AddField(
            model_name='autor',
            name='aktualna_funkcja',
            field=models.ForeignKey(on_delete=models.CASCADE, related_name='aktualna_funkcja', blank=True, to='bpp.Funkcja_Autora', null=True),
        ),
        RunPython(
            lambda *args, **kw: load_custom_sql("0046_ostatnia_jednostka_trigger"),
            lambda *args, **kw: load_custom_sql("0046_ostatnia_jednostka_trigger_remove.sql"))
    ]



# Generated by Django 3.0.4 on 2020-05-14 11:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0210_uczelnia_sortuj_jednostki_alfabetycznie"),
    ]

    operations = [
        migrations.AddField(
            model_name="praca_doktorska",
            name="oznaczenie_wydania",
            field=models.CharField(blank=True, max_length=400, null=True),
        ),
        migrations.AddField(
            model_name="praca_habilitacyjna",
            name="oznaczenie_wydania",
            field=models.CharField(blank=True, max_length=400, null=True),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte",
            name="oznaczenie_wydania",
            field=models.CharField(blank=True, max_length=400, null=True),
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("integrator2", "0003_django110_py3k"),
    ]

    operations = [
        migrations.AlterField(
            model_name="listaministerialnaelement",
            name="moze_byc_zintegrowany_automatycznie",
            field=models.BooleanField(default=None, null=True),
        ),
    ]



# Generated by Django 3.0.11 on 2021-03-23 00:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rozbieznosci_if", "0002_auto_20210323_0106"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="ignorujrozbieznoscif",
            options={"verbose_name": "ignorowanie rozbieżności impact factor"},
        ),
        migrations.AlterField(
            model_name="ignorujrozbieznoscif",
            name="created_on",
            field=models.DateTimeField(auto_now_add=True, verbose_name="Utworzono"),
        ),
        migrations.AlterField(
            model_name="ignorujrozbieznoscif",
            name="object_id",
            field=models.PositiveIntegerField(db_index=True, verbose_name="Rekord"),
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-16 18:14

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0206_auto_20200315_2212"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        )
    ]



# Generated by Django 3.0.14 on 2021-08-17 22:16
import warnings

from django.core.paginator import Paginator
from django.db import migrations

from bpp.util import pbar


def value(elem, *path, return_none=False):
    v = None
    if elem.versions:
        for _elem in elem.versions:
            if _elem["current"]:
                v = _elem
                break

    # v = elem.current_version

    if v is None:
        warnings.warn(
            f"Model {elem.__class__} with id {elem.mongoId} has NO current_version!"
        )
        if return_none:
            return
        return "[brak current_version]"

    for elem in path:
        if elem in v:
            v = v[elem]
        else:
            if return_none:
                return None
            return f"[brak {elem}]"
    return v


def value_or_none(elem, *path):
    return value(elem, *path, return_none=True)


MAX_TEXT_FIELD_LENGTH = 512


def _pull_up_on_save(elem, pull_up_on_save):
    for attr in pull_up_on_save:
        v = value_or_none(elem, "object", attr)
        if v is not None:
            if isinstance(v, str):
                if len(v) >= MAX_TEXT_FIELD_LENGTH:
                    v = v[:MAX_TEXT_FIELD_LENGTH]
        setattr(elem, attr, v)


def rebuild_table(model, puos):
    queryset = model.objects.all().only("pk", "versions").order_by("pk")
    paginator = Paginator(queryset, 1000)
    for page in pbar(paginator.page_range):
        for elem in paginator.get_page(page).object_list:
            _pull_up_on_save(elem, puos)
            elem.save(update_fields=puos)


def rebuild(apps, schema_editor):
    for model, puos in [
        (
            apps.get_model("pbn_api", "Publication"),
            ["year"],
        ),
    ]:
        rebuild_table(model, puos)


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0026_auto_20210816_0815"),
    ]

    operations = [
        migrations.RunPython(rebuild, migrations.RunPython.noop),
    ]



from uuid import uuid4

from django import template
from django.template import Node, TemplateSyntaxError
from django.template.defaultfilters import stringfilter

register = template.Library()


@register.filter(name="message_level_to_css_class")
@stringfilter
def message_level_to_css_class(value):
    from django.contrib.messages import constants

    return constants.DEFAULT_TAGS.get(int(value))



import openpyxl
from _decimal import Decimal, InvalidOperation
from openpyxl.utils.exceptions import InvalidFileException

from import_common.core import matchuj_autora, matchuj_jednostke, matchuj_wydzial
from import_common.exceptions import (
    BadNoOfSheetsException,
    HeaderNotFoundException,
    ImproperFileException,
)
from import_dyscyplin.models import Import_Dyscyplin_Row

from bpp.models import Jednostka


def przeanalizuj_plik_xls(sciezka, parent):
    """
    :param sciezka:
    :return: (success:boolean, message)
    """

    # Otwórz plik i sprawdź, czy w ogóle działa...
    try:
        f = openpyxl.load_workbook(sciezka)
    except InvalidFileException as e:
        raise ImproperFileException(e)

    # Sprawdź, ile jest skoroszytów
    if len(f.worksheets) != 1:
        raise BadNoOfSheetsException()

    s = f.worksheets[0]

    if parent.kolumna_set.all().count() == 0:
        raise HeaderNotFoundException()

    naglowek = [k.rodzaj_pola for k in parent.kolumna_set.all()]

    wydzial_cache = {}
    jednostka_cache = {}

    for n, row in enumerate(
        s.iter_rows(
            min_row=parent.wiersz_naglowka + 1, max_row=s.max_row, values_only=True
        ),
        parent.wiersz_naglowka + 1,
    ):
        original = dict(zip(naglowek, [elem for elem in row]))

        if original["nazwisko"] is None or (not original["nazwisko"].strip()):
            continue

        try:
            # templatka wymaga
            original["nazwa_jednostki"] = original["nazwa jednostki"]
        except KeyError:
            original["nazwa_jednostki"] = None

        try:
            wydzial = wydzial_cache.get(original["wydział"])
        except KeyError:
            wydzial = None

        if not wydzial:
            try:
                wydzial = wydzial_cache[original["wydział"]] = matchuj_wydzial(
                    original["wydział"]
                )
            except KeyError:
                pass

        try:
            jednostka = jednostka_cache.get(original["nazwa jednostki"])
        except KeyError:
            jednostka = None

        if not jednostka:
            try:
                jednostka = jednostka_cache[
                    original["nazwa jednostki"]
                ] = matchuj_jednostke(original["nazwa jednostki"])
            except (
                KeyError,
                Jednostka.DoesNotExist,
                Jednostka.MultipleObjectsReturned,
            ):
                jednostka = None

        autor = matchuj_autora(
            original["imię"],
            original["nazwisko"],
            jednostka=jednostka,
            orcid=original.get("orcid", None),
            pbn_id=original.get("pbn_id", None),
            tytul_str=original.get("tytuł"),
        )

        if autor is None:
            pass

        bledny = False

        for elem in ["procent dyscypliny", "procent subdyscypliny"]:
            v = original.get(elem)

            if v is None:
                continue

            if str(v) == "":
                original[elem] = None
                continue

            try:
                original[elem] = Decimal(original.get(elem))
            except (TypeError, InvalidOperation):
                original[elem] = None
                bledny = True

        # Akceptujemy pliki bez pierwszej dyscypliny -- taki zapis w pliku oznacza,
        # że autorowi dyscyplina jest kasowana. Zatem, poniższy kod zostaje usunięty:
        # if original.get("dyscyplina").strip() == "":
        #     bledny = True

        i = Import_Dyscyplin_Row(
            row_no=n,
            parent=parent,
            original=original,
            nazwisko=original["nazwisko"],
            imiona=original["imię"],
            nazwa_jednostki=original.get("nazwa_jednostki"),
            jednostka=jednostka,
            nazwa_wydzialu=original.get("wydział"),
            wydzial=wydzial,
            autor=autor,
            dyscyplina=original.get("dyscyplina"),
            kod_dyscypliny=original.get("kod dyscypliny"),
            procent_dyscypliny=original.get("procent dyscypliny"),
            subdyscyplina=original.get("subdyscyplina"),
            kod_subdyscypliny=original.get("kod subdyscypliny"),
            procent_subdyscypliny=original.get("procent subdyscypliny"),
        )

        if i.dyscyplina_naukowa is None:
            i.procent_dyscypliny = None

        if i.subdyscyplina_naukowa is None:
            i.procent_subdyscypliny = None

        if autor is None:
            i.stan = Import_Dyscyplin_Row.STAN.BLEDNY
            i.info = "Nie można dopasować autora"
        else:
            if i.dyscyplina_naukowa is None and i.subdyscyplina_naukowa is not None:
                i.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                i.info = "Dyscyplina pusta, subdyscyplina ustawiona."

            elif bledny:
                i.stan = Import_Dyscyplin_Row.STAN.BLEDNY
                i.info = (
                    "niepoprawna wartość w polu procent dyscypliny lub subdyscypliny"
                )

        i.save()

    return (True, "przeanalizowano %i rekordow" % (n - parent.wiersz_naglowka))



from django.contrib import admin

from bpp.models.szablondlaopisubibliograficznego import SzablonDlaOpisuBibliograficznego
from bpp.util import rebuild_instances_of_models


@admin.register(SzablonDlaOpisuBibliograficznego)
class SzablonDlaOpisuBibliograficznegoAdmin(admin.ModelAdmin):
    list_display = ["model", "template"]
    empty_value_display = "(każdy)"

    def save_model(self, request, obj: SzablonDlaOpisuBibliograficznego, form, change):
        super(SzablonDlaOpisuBibliograficznegoAdmin, self).save_model(
            request, obj, form, change
        )
        rebuild_instances_of_models(obj.get_models_for_this_szablon())

    def delete_model(self, request, obj: SzablonDlaOpisuBibliograficznego):
        super(SzablonDlaOpisuBibliograficznegoAdmin, self).delete_model(request, obj)
        rebuild_instances_of_models(
            SzablonDlaOpisuBibliograficznego.objects.all_templated_models
        )



from model_bakery import baker

from zglos_publikacje.admin.filters import WydzialJednostkiPierwszegoAutora
from zglos_publikacje.models import Zgloszenie_Publikacji

from bpp.models import Typ_Odpowiedzialnosci


def test_WydzialJednostkiPierwszegoAutora_queryset(
    aktualna_jednostka,
    druga_aktualna_jednostka,
    wydzial,
    drugi_wydzial,
    autor_jan_kowalski,
    autor_jan_nowak,
    typy_odpowiedzialnosci,
    rf,
):
    zgloszenie = baker.make(Zgloszenie_Publikacji)
    zgloszenie.zgloszenie_publikacji_autor_set.create(
        autor=autor_jan_kowalski,
        jednostka=aktualna_jednostka,
        kolejnosc=0,
        typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.first(),
        rok=2022,
    )
    zgloszenie.zgloszenie_publikacji_autor_set.create(
        autor=autor_jan_nowak,
        jednostka=druga_aktualna_jednostka,
        kolejnosc=1,
        typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.first(),
        rok=2022,
    )

    params = {"wydz1a": str(wydzial.pk)}
    req = rf.get("/", params)
    filtr = WydzialJednostkiPierwszegoAutora(req, params, Zgloszenie_Publikacji, None)
    qs = filtr.queryset(req, Zgloszenie_Publikacji.objects.all())
    assert qs.count() == 1

    params = {"wydz1a": str(drugi_wydzial.pk)}
    req = rf.get("/", params)
    filtr = WydzialJednostkiPierwszegoAutora(req, params, Zgloszenie_Publikacji, None)
    qs = filtr.queryset(req, Zgloszenie_Publikacji.objects.all())
    assert qs.count() == 0



from ..models import Kierunek_Studiow
from .core import BaseBppAdminMixin, RestrictDeletionToAdministracjaGroupMixin

from django.contrib import admin


@admin.register(Kierunek_Studiow)
class Kierunek_StudiowAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    list_display_links = ["nazwa"]
    list_display = ["nazwa", "skrot", "wydzial"]
    search_fields = ["nazwa", "skrot", "wydzial__nazwa"]
    fields = ["nazwa", "skrot", "wydzial", "opis", "adnotacje"]



# Generated by Django 3.0.11 on 2021-04-25 23:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0261_auto_20210426_0124"),
    ]

    operations = [
        migrations.AddField(
            model_name="bppuser",
            name="pbn_token",
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
    ]



# -*- encoding: utf-8 -*-

from selenium.webdriver.support.wait import WebDriverWait

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import pytest

from bpp.models import Wydawnictwo_Ciagle
from bpp.models.patent import Patent
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from bpp.tests import any_ciagle
from bpp.tests.util import any_patent, any_zwarte, assertPopupContains

from django_bpp.selenium_util import SHORT_WAIT_TIME, wait_for, wait_for_page_load

ID = "id_tytul_oryginalny"

pytestmark = [pytest.mark.slow, pytest.mark.selenium]


def test_admin_wydawnictwo_ciagle_toz(admin_browser, asgi_live_server):
    Wydawnictwo_Ciagle.objects.all().delete()
    c = any_ciagle(informacje="TO INFORMACJE")

    admin_browser.visit(
        asgi_live_server.url
        + reverse("admin:bpp_wydawnictwo_ciagle_change", args=(c.pk,))
    )

    wcc = Wydawnictwo_Ciagle.objects.count
    assert wcc() == 1

    toz = admin_browser.find_by_id("toz")

    toz.click()
    assertPopupContains(admin_browser, "Utworzysz kopię tego rekordu")
    wait_for(lambda: wcc() == 2)


def test_admin_wydawnictwo_zwarte_toz(admin_browser, asgi_live_server):
    c = any_zwarte(informacje="TO INFOMRACJE")

    admin_browser.visit(
        asgi_live_server.url
        + reverse("admin:bpp_wydawnictwo_zwarte_change", args=(c.pk,))
    )

    wcc = Wydawnictwo_Zwarte.objects.count
    assert wcc() == 1

    toz = admin_browser.find_by_id("toz")
    toz.click()
    assertPopupContains(admin_browser, "Utworzysz kopię tego rekordu")
    wait_for(lambda: wcc() == 2)


def test_admin_wydawnictwo_ciagle_tamze(admin_browser, asgi_live_server):
    c = any_ciagle(informacje="TO INFORMACJE", uwagi="te uwagi", www="te www")
    admin_browser.visit(
        asgi_live_server.url
        + reverse("admin:bpp_wydawnictwo_ciagle_change", args=(c.pk,))
    )

    tamze = admin_browser.find_by_id("tamze")

    with wait_for_page_load(admin_browser):
        tamze.click()

    assert "Dodaj wydawnictwo" in admin_browser.html

    for elem in ["TO INFORMACJE", "te uwagi"]:
        assert elem in admin_browser.html, "BRAK %r" % elem
    assert "te www" not in admin_browser.html

    assert "te www" not in admin_browser.html


def test_admin_wydawnictwo_zwarte_tamze(admin_browser, asgi_live_server, wydawca):
    c = any_zwarte(
        informacje="TO INFORMACJE",
        uwagi="te uwagi",
        miejsce_i_rok="te miejsce i rok",
        wydawca=wydawca,
        wydawca_opis="te wydawnictwo",
        www="ten adres WWW",
        isbn="Z_ISBN",
        e_isbn="E_ISBN",
    )
    admin_browser.visit(
        asgi_live_server.url
        + reverse("admin:bpp_wydawnictwo_zwarte_change", args=(c.pk,))
    )
    tamze = admin_browser.find_by_id("tamze")
    tamze.click()
    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        lambda driver: "Dodaj wydawnictwo" in driver.page_source
    )
    for elem in [
        "TO INFORMACJE",
        "te uwagi",
        "te miejsce i rok",
        "te wydawnictwo",
        "Z_ISBN",
        "E_ISBN",
        "Wydawca Testowy",
    ]:
        assert elem in admin_browser.html, "BRAK %r" % elem
    assert "ten adres WWW" not in admin_browser.html


def test_admin_patent_toz(admin_browser, asgi_live_server):
    c = any_patent(informacje="TO INFORMACJE")
    admin_browser.visit(
        asgi_live_server.url + reverse("admin:bpp_patent_change", args=(c.pk,))
    )

    wcc = Patent.objects.count
    assert wcc() == 1

    toz = admin_browser.find_by_id("toz")
    toz.click()

    assertPopupContains(admin_browser, "Utworzysz kopię tego rekordu")
    WebDriverWait(admin_browser, SHORT_WAIT_TIME).until(
        lambda driver: admin_browser.is_element_present_by_id("navigation-menu")
    )

    wait_for(lambda: wcc() == 2)


def test_admin_patent_tamze(admin_browser, asgi_live_server):
    c = any_patent(informacje="TO INFORMACJE")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(
            asgi_live_server.url + reverse("admin:bpp_patent_change", args=(c.pk,))
        )

    with wait_for_page_load(admin_browser):
        admin_browser.execute_script("document.getElementById('tamze').click()")

    assert admin_browser.find_by_id("id_informacje").value == "TO INFORMACJE"



# Generated by Django 3.0.11 on 2021-02-28 18:16

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_pracownikow", "0002_importpracownikowrow"),
    ]

    operations = [
        migrations.AddField(
            model_name="importpracownikowrow",
            name="integracja_mozliwa",
            field=models.NullBooleanField(),
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="parent",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="row_set",
                to="import_pracownikow.ImportPracownikow",
            ),
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-08 11:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0324_django32"),
    ]

    operations = [
        migrations.AlterField(
            model_name="autor_jednostka",
            name="podstawowe_miejsce_pracy",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="patent",
            name="wdrozenie",
            field=models.BooleanField(
                blank=True, default=None, null=True, verbose_name="Wdrożenie"
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="opl_pub_other",
            field=models.BooleanField(
                blank=True,
                default=None,
                null=True,
                verbose_name="Inne środki finansowe",
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="opl_pub_other",
            field=models.BooleanField(
                blank=True,
                default=None,
                null=True,
                verbose_name="Inne środki finansowe",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="opl_pub_other",
            field=models.BooleanField(
                blank=True,
                default=None,
                null=True,
                verbose_name="Inne środki finansowe",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="opl_pub_other",
            field=models.BooleanField(
                blank=True,
                default=None,
                null=True,
                verbose_name="Inne środki finansowe",
            ),
        ),
    ]



from abc import abstractmethod

from dal import autocomplete
from django import http
from django.db.models.query_utils import Q
from sentry_sdk import capture_exception

from pbn_api.client import PBNClient
from pbn_api.exceptions import AccessDeniedException
from pbn_api.integrator import ACTIVE, zapisz_mongodb
from pbn_api.models import Journal, Publication, Scientist

from django.contrib.auth.mixins import LoginRequiredMixin

from django.utils.translation import ugettext_lazy as _

from bpp import const
from bpp.const import PBN_UID_LEN
from bpp.models import Uczelnia


class BasePBNAutocomplete(LoginRequiredMixin, autocomplete.Select2QuerySetView):
    """
    Ta klasa robi wyszukiwanie zadeklarowanego modelu z PBN, jednocześnie pozwala
    też utworzyć w bazie danych taki model - na podstawie mongoID.
    """

    create_field = "mongoId"

    @property
    def pbn_api_model(self):
        raise NotImplementedError("Please override this in a subclass")

    @property
    def sort_order(self):
        raise NotImplementedError("Please override this in a subclass")

    @abstractmethod
    def filter_queryset(self, qs, word):
        raise NotImplementedError("Please override in a subclass")

    @abstractmethod
    def fetch_pbn_data(self, client, query):
        raise NotImplementedError("Please override in a subclass")

    def get_create_option(self, context, q):
        """Form the correct create_option to append to results."""
        create_option = []
        display_create_option = False
        if self.create_field and q:
            page_obj = context.get("page_obj", None)
            if (page_obj is None or page_obj.number == 1) and len(
                q.strip()
            ) == PBN_UID_LEN:
                display_create_option = True

            # Don't offer to create a new option if a
            # case-insensitive) identical one already exists
            existing_options = (
                self.get_result_label(result).lower()
                for result in context["object_list"]
            )
            if q.lower() in existing_options:
                display_create_option = False
            if Publication.objects.filter(pk=q.lower()).exists():
                display_create_option = False

        if display_create_option and self.has_add_permission(self.request):
            create_option = [
                {
                    "id": q,
                    "text": _('Pobierz rekord o UID "%(new_value)s" z serwera PBNu')
                    % {"new_value": q},
                    "create_id": True,
                }
            ]
        return create_option

    def create_object(self, text):
        uczelnia = Uczelnia.objects.get_for_request(self.request)
        client = uczelnia.pbn_client(self.request.user.pbn_token)
        return zapisz_mongodb(self.fetch_pbn_data(client, text), self.pbn_api_model)

    def post(self, request, *args, **kwargs):
        """Create an object given a text after checking permissions."""
        if not self.has_add_permission(request):
            return http.HttpResponseForbidden()

        text = request.POST.get("text", None)

        if text is None:
            return http.HttpResponseBadRequest()

        try:
            result = self.create_object(text)
        except AccessDeniedException:
            return http.JsonResponse(
                {
                    "id": None,
                    "text": "Brak dostępu. Autoryzuj się w PBN API za pomocą opcji na głównej stronie. ",
                }
            )

        except Exception as e:
            # Zaloguj wyjątek do Sentry
            capture_exception(e)
            return http.HttpResponseBadRequest(str(e))

        return http.JsonResponse(
            {
                "id": result.pk,
                "text": self.get_selected_result_label(result),
            }
        )

    def get_base_queryset(self):
        return self.pbn_api_model.objects.filter(status=ACTIVE)

    def separate_words(self, q):
        return [word.strip() for word in q.strip().split(" ") if word.strip()]

    def get_queryset(self):
        qs = self.get_base_queryset()

        self.q = self.q.strip()

        if self.q:
            if len(self.q) == const.PBN_UID_LEN and self.q.find(" ") == -1:
                # podany parametr ma długość PBN_UID, więc szukamy po PBN UID:
                qs = qs.filter(pk=self.q)
            else:
                words = self.separate_words(self.q)
                for word in words:
                    qs = self.filter_queryset(qs, word)
        return qs.order_by(*self.sort_order)


class PublicationAutocomplete(BasePBNAutocomplete):

    pbn_api_model = Publication
    sort_order = ("title", "-year")

    def fetch_pbn_data(self, client, query):
        return client.get_publication_by_id(query)

    def filter_queryset(self, qs, word):
        return qs.filter(
            Q(title__istartswith=self.q.strip())
            | Q(title__icontains=word)
            | Q(isbn__exact=word)
            | Q(doi__icontains=word)
        )


class ScientistAutocomplete(BasePBNAutocomplete):

    pbn_api_model = Scientist
    sort_order = ("from_institution_api",)  # "lastName", "name")

    def fetch_pbn_data(self, client, query):
        return client.get_person_by_id(query)

    def filter_queryset(self, qs, word):
        return qs.filter(
            Q(lastName__istartswith=self.q.strip())
            | Q(name__icontains=word)
            | Q(pbnId__exact=word)
            | Q(orcid__icontains=word)
        )


class JournalAutocomplete(BasePBNAutocomplete):

    pbn_api_model = Journal
    sort_order = ("title", "mniswId")

    def fetch_pbn_data(self, client: PBNClient, query):
        return client.get_journal_by_id(query)

    def filter_queryset(self, qs, word):
        return qs.filter(
            Q(title__istartswith=self.q.strip())
            | Q(title__icontains=word)
            | Q(issn__exact=word)
            | Q(eissn__exact=word)
            | Q(websiteLink__icontains=word)
        )



# Generated by Django 3.0.14 on 2021-08-20 14:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0289_data_oswiadczenia"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bppmultiseekvisibility",
            name="field_name",
            field=models.CharField(
                db_index=True,
                max_length=50,
                unique=True,
                verbose_name="Systemowa nazwa pola",
            ),
        ),
        migrations.AlterField(
            model_name="bppmultiseekvisibility",
            name="label",
            field=models.CharField(max_length=200, verbose_name="Nazwa pola"),
        ),
        migrations.AlterField(
            model_name="bppmultiseekvisibility",
            name="public",
            field=models.BooleanField(
                default=True, verbose_name="Widoczne dla niezalogowanych"
            ),
        ),
        migrations.AlterField(
            model_name="bppmultiseekvisibility",
            name="sort_order",
            field=models.PositiveSmallIntegerField(
                default=0, verbose_name="Kolejność sortowania"
            ),
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_dyscyplin", "0019_drop_pesel_md5"),
    ]

    operations = [
        migrations.AlterField(
            model_name="import_dyscyplin_row",
            name="original",
            field=models.JSONField(
                encoder=django.core.serializers.json.DjangoJSONEncoder
            ),
        ),
    ]



import json

from django.db import transaction
from django.db.models import Q

from formdefaults.util import full_name


def update_form_db_repr(form_instance, form_repr, user=None):
    """
    Aktualizuje reprezentację formularza w bazie danych:

    1) usuwa wszystkie pola które są w bazie a których nie ma w formularzu
    2) tworzy pola które są w formularzu wraz z domyślnymi wartościami

    :type form_repr: formdefaults.models.FormRepresentation
    :type form_instance: django.forms.Form
    """

    form_fields = form_instance.fields
    form_fields_names = form_fields.keys()

    db_fields = form_repr.fields_set.all()

    # Usuwanie pól, których nie ma w formularzu:
    db_fields.filter(~Q(name__in=form_fields_names)).delete()

    db_fields_dict = {db_field.name: db_field for db_field in db_fields}

    # Przeleć listę pól, w razie potrzeby aktualizując typ lub etykietę
    for no, field_name in enumerate(form_fields_names):
        form_field = form_fields[field_name]

        db_field = db_fields_dict.get(field_name)

        created = False
        if db_field is None:
            created = True
            db_field = db_fields.create(
                parent=form_repr,
                name=field_name,
                klass=full_name(form_field),
                label=form_field.label or field_name.replace("_", " ").capitalize(),
                order=no,
            )

        updated = False
        if db_field.label != form_field.label:
            db_field.label = form_field.label
            updated = True

        if full_name(form_field) != db_field.klass:
            db_field.klass = full_name(form_field)
            updated = True

        if updated:
            db_field.save()

        # Sprawdź, czy wartość domyślna pola może być zapisana w bazie, tzn. czy
        # poddaje się 'testowi' zakodowania jej w formacie JSON. Na tym etapie jeżeli
        # wartość domyślna jest liczona np za pomocą funkcji, to system przejdzie
        # do kolejnego pola, bez tworzenia wartości domyślnej w bazie danych.
        form_field_value = form_field.initial
        try:
            json.dumps(form_field_value)
        except TypeError:
            if not created:
                db_field.delete()
            continue

        if created:
            # Reprezentację pola w bazie danych własnie utworzono, więc z tej okazji
            # zapisz do bazy danych wartość domyślną tego pola w momencie tworzenia go:
            form_repr.values_set.create(
                field=db_field, value=form_field_value, user=user
            )

        # Jeżeli jest podany użytkownik to sprawdź, czy dla niego jest wpis w bazie danych;
        # Jeżeli tego wpisu nie ma to utwórz go z wartością domyślną
        if user is not None:
            user_value, created = form_repr.values_set.get_or_create(
                field=db_field, user=user
            )

            if created:
                user_value.value = form_field_value
                user_value.save()


@transaction.atomic
def get_form_defaults(form_instance, label=None, user=None, update_db_repr=True):
    fn = full_name(form_instance)

    from formdefaults.models import FormRepresentation

    form_repr, crt = FormRepresentation.objects.get_or_create(full_name=fn)

    if update_db_repr:
        if label is not None:
            if form_repr.label != label:
                form_repr.label = label
                form_repr.save()

        # Automatyczne tworzenie reprezentacji bazodanowej formularza w tej
        # funkcji NIE przewiduje tworzenia oddzielnych ustaleń dla danego użytkownika
        update_form_db_repr(form_instance, form_repr, user=None)

    # Weź listę domyślnych wartości (czyli takich gdzie user=None)
    values = dict(
        [
            (qs["field__name"], qs["value"])
            for qs in form_repr.values_set.filter(user=None)
            .select_related("field__name")
            .values("field__name", "value")
        ]
    )

    # Weź listę wartości dla danego użytkownika
    if user is not None:
        user_values = dict(
            [
                (qs["field__name"], qs["value"])
                for qs in form_repr.values_set.filter(user=user)
                .select_related("field__name")
                .values("field__name", "value")
            ]
        )
        values.update(user_values)

    values.update(
        {
            "formdefaults_pre_html": form_repr.html_before,
            "formdefaults_post_html": form_repr.html_after,
        }
    )

    return values



# -*- encoding: utf-8 -*-

import pytest

from bpp.models.konferencja import Konferencja


@pytest.mark.django_db
def test_konferencja___str__():
    konf = Konferencja.objects.create(
        nazwa="Konferencja", baza_scopus=True, baza_wos=True, baza_inna="Hej"
    )

    assert str(konf) == "Konferencja [Scopus] [WoS] [Hej]"



from django.conf.urls import url

from import_dyscyplin.views import CreateImport_Dyscyplin, DetailImport_Dyscyplin, UruchomPrzetwarzanieImport_Dyscyplin, \
    ListImport_Dyscyplin, UsunImport_Dyscyplin, API_Do_IntegracjiView, API_Nie_Do_IntegracjiView, API_Zintegrowane, \
    UruchomIntegracjeImport_DyscyplinView, KolumnyImport_Dyscyplin, UruchomTworzenieKolumnImport_Dyscyplin

urlpatterns = [
    url(
        r'^$',
        ListImport_Dyscyplin.as_view(),
        name='index'
    ),

    url(
        r'^create/$',
        CreateImport_Dyscyplin.as_view(),
        name='create'
    ),

    url(
        r'^detail/(?P<pk>\d+)/$',
        DetailImport_Dyscyplin.as_view(),
        name='detail'
    ),

    url(
        r'^okresl-kolumny/(?P<pk>\d+)/$',
        KolumnyImport_Dyscyplin.as_view(),
        name='okresl_kolumny'
    ),

    url(
        r'^api/stworz-kolumny/(?P<pk>\d+)/$',
        UruchomTworzenieKolumnImport_Dyscyplin.as_view(),
        name='stworz_kolumny'
    ),

    url(
        r'^api/przetwarzaj/(?P<pk>\d+)/$',
        UruchomPrzetwarzanieImport_Dyscyplin.as_view(),
        name='przetwarzaj'
    ),

    url(
        r'^api/integruj/(?P<pk>\d+)/$',
        UruchomIntegracjeImport_DyscyplinView.as_view(),
        name='integruj'
    ),

    url(
        r'^api/usun/(?P<pk>\d+)/$',
        UsunImport_Dyscyplin.as_view(),
        name='usun'
    ),

    url(
        r'^api/do_integracji/(?P<pk>\d+)/$',
        API_Do_IntegracjiView.as_view(),
        name='api_do_integracji'
    ),

    url(
        r'^api/nie_do_integracji/(?P<pk>\d+)/$',
        API_Nie_Do_IntegracjiView.as_view(),
        name='api_nie_do_integracji'
    ),
    url(
        r'^api/zintegrowane/(?P<pk>\d+)/$',
        API_Zintegrowane.as_view(),
        name='api_zintegrowane'
    ),

]



# Generated by Django 3.0.11 on 2021-04-21 21:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0008_auto_20210406_0530"),
        ("bpp", "0258_auto_20210407_2320"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="jezyk",
            name="skrot_dla_pbn",
        ),
        migrations.AddField(
            model_name="jezyk",
            name="pbn_uid",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="pbn_api.Language",
            ),
        ),
        migrations.AlterField(
            model_name="autor",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="jednostka",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="patent",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
        migrations.AlterField(
            model_name="wydzial",
            name="pbn_id",
            field=models.IntegerField(
                blank=True,
                db_index=True,
                help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
                null=True,
                unique=True,
                verbose_name="[Przestarzałe] Identyfikator PBN",
            ),
        ),
    ]



import pytest
from model_bakery import baker

from import_common.core import matchuj_dyscypline
from import_dyscyplin.core import matchuj_autora, matchuj_jednostke, matchuj_wydzial

from bpp.models import Autor, Dyscyplina_Naukowa, Jednostka, Tytul, Wydzial


@pytest.mark.parametrize(
    "szukany_string",
    [
        "II Lekarski",
        "II Lekarski ",
        "ii lekarski",
        "   ii lekarski  ",
    ],
)
def test_matchuj_wydzial(szukany_string, db):
    baker.make(Wydzial, nazwa="I Lekarski")
    w2 = baker.make(Wydzial, nazwa="II Lekarski")

    assert matchuj_wydzial(szukany_string) == w2


@pytest.mark.parametrize(
    "szukany_string",
    ["Jednostka Pierwsza", "  Jednostka Pierwsza  \t", "jednostka pierwsza"],
)
def test_matchuj_jednostke(szukany_string, uczelnia, wydzial, db):
    j1 = baker.make(
        Jednostka, nazwa="Jednostka Pierwsza", wydzial=wydzial, uczelnia=uczelnia
    )
    baker.make(
        Jednostka,
        nazwa="Jednostka Pierwsza i Jeszcze",
        wydzial=wydzial,
        uczelnia=uczelnia,
    )

    assert matchuj_jednostke(szukany_string) == j1


def test_matchuj_autora_imiona_nazwisko(autor_jan_nowak):
    a = matchuj_autora("Jan", "Nowak", jednostka=None)
    assert a == autor_jan_nowak


@pytest.mark.django_db
def test_matchuj_autora_po_aktualnej_jednostce():
    j1 = baker.make(Jednostka)
    j2 = baker.make(Jednostka)

    a1 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
    a1.dodaj_jednostke(j1)

    a2 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
    a2.dodaj_jednostke(j2)

    a = matchuj_autora(imiona="Jan", nazwisko="Kowalski", jednostka=None)
    assert a is None

    a = matchuj_autora(imiona="Jan", nazwisko="Kowalski", jednostka=j1)
    assert a == a1

    a = matchuj_autora(imiona="Jan", nazwisko="Kowalski", jednostka=j2)
    assert a == a2


@pytest.mark.django_db
def test_matchuj_autora_po_jednostce():
    j1 = baker.make(Jednostka)
    j2 = baker.make(Jednostka)

    a1 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
    a1.dodaj_jednostke(j1)
    a1.aktualna_jednostka = None
    a1.save()

    a2 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
    a2.dodaj_jednostke(j2)
    a2.aktualna_jednostka = None
    a2.save()

    a = matchuj_autora(imiona="Jan", nazwisko="Kowalski", jednostka=j1)
    assert a == a1

    a = matchuj_autora(imiona="Jan", nazwisko="Kowalski", jednostka=j2)
    assert a == a2


@pytest.mark.django_db
def test_matchuj_autora_po_tytule():
    t = Tytul.objects.create(nazwa="prof hab", skrot="lol.")

    baker.make(Jednostka)

    a1 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
    a1.tytul = t
    a1.save()

    baker.make(Autor, imiona="Jan", nazwisko="Kowalski")

    a = matchuj_autora(
        imiona="Jan",
        nazwisko="Kowalski",
    )
    assert a is None

    a = matchuj_autora(imiona="Jan", nazwisko="Kowalski", tytul_str="lol.")
    assert a == a1


@pytest.mark.django_db
def test_matchuj_autora_tytul_bug(jednostka):
    matchuj_autora("Kowalski", "Jan", jednostka, tytul_str="Doktur")
    assert True


@pytest.mark.django_db
def test_matchuj_dyscypline():
    d = Dyscyplina_Naukowa.objects.create(nazwa="foo", kod="4.3")
    assert matchuj_dyscypline("403_0", "aoijsdf") == d
    assert matchuj_dyscypline("403", "aoijsdf") == d
    assert matchuj_dyscypline("4.3", "aoijsdf") == d
    assert matchuj_dyscypline("nieno", "foo") == d



# -*- encoding: utf-8 -*-

from zope.component import getGlobalSiteManager
from celeryui.interfaces import IReport, IWebTask, IReportMaker

_REGISTRY = {}


class ReportAdapter:
    def __init__(self, original):
        self.original = original


def registerAdapter(function, klass):
    """This registers an adapter for reports with .function attribute
    to a given class."""
    _REGISTRY[function] = klass


def reportAdapter(original, *arg, **kw):
    return _REGISTRY.get(original.function)(original)


getGlobalSiteManager().registerAdapter(reportAdapter, [IReport], IWebTask)
getGlobalSiteManager().registerAdapter(reportAdapter, [IReport], IReportMaker)





# Generated by Django 3.0.14 on 2021-09-05 18:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0291_kod_dyscypliny_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="patent_autor",
            name="przypieta",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text='Możesz odznaczyć, żeby "odpiąć" dyscyplinę od tego autora. Dyscyplina "odpięta" nie będzie\n        wykazywana do PBN, ale będzie pojawiała się np. w raportach i innych zestawieniach. ',
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle_autor",
            name="przypieta",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text='Możesz odznaczyć, żeby "odpiąć" dyscyplinę od tego autora. Dyscyplina "odpięta" nie będzie\n        wykazywana do PBN, ale będzie pojawiała się np. w raportach i innych zestawieniach. ',
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte_autor",
            name="przypieta",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text='Możesz odznaczyć, żeby "odpiąć" dyscyplinę od tego autora. Dyscyplina "odpięta" nie będzie\n        wykazywana do PBN, ale będzie pojawiała się np. w raportach i innych zestawieniach. ',
            ),
        ),
    ]



# Generated by Django 3.0.7 on 2020-08-06 06:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0215_auto_20200806_0146"),
        ("bpp", "0220_auto_20200728_0011"),
    ]

    operations = []



import pytest
from django.urls import reverse
from model_bakery import baker

from raport_slotow.models.uczelnia import (
    RaportSlotowUczelnia,
    RaportSlotowUczelniaWiersz,
)

from django.utils import timezone


def test_ListaRaportSlotowUczelnia(admin_client, admin_user):

    for no in range(20):
        baker.make(
            RaportSlotowUczelnia, owner=admin_user, od_roku=2000 + no, do_roku=2000 + no
        )

    assert RaportSlotowUczelnia.objects.count() == 20

    admin_client.get(reverse("raport_slotow:lista-raport-slotow-uczelnia"))

    assert RaportSlotowUczelnia.objects.count() == 10


def test_SzczegolyRaportuSlotowUczelnia(admin_client, admin_user):
    rsu = baker.make(RaportSlotowUczelnia, owner=admin_user)
    admin_client.get(
        reverse("raport_slotow:raportslotowuczelnia-details", args=(rsu.pk,))
    )


@pytest.mark.django_db
@pytest.mark.parametrize("dziel_na_jednostki_i_wydzialy", [True, False])
def test_SzczegolyRaportuSlotowUczelniaListaRekordow(
    admin_client, admin_user, dziel_na_jednostki_i_wydzialy
):
    rsu = baker.make(
        RaportSlotowUczelnia,
        owner=admin_user,
        dziel_na_jednostki_i_wydzialy=dziel_na_jednostki_i_wydzialy,
    )
    baker.make(RaportSlotowUczelniaWiersz, parent=rsu)

    admin_client.get(
        reverse(
            "raport_slotow:raportslotowuczelnia-results",
            args=(rsu.pk,),
        )
    )

    res = admin_client.get(
        reverse(
            "raport_slotow:raportslotowuczelnia-results",
            args=(rsu.pk,),
        )
        + "?_export=xlsx"
    )
    assert res["content-type"] == "application/vnd.ms-excel"


def test_RegenerujRaportuSlotowUczelnia(
    admin_client,
    admin_user,
):
    rsu = baker.make(
        RaportSlotowUczelnia,
        owner=admin_user,
        finished_successfully=True,
        finished_on=timezone.now(),
    )
    first_finished_on = rsu.finished_on
    baker.make(RaportSlotowUczelniaWiersz, parent=rsu)

    admin_client.get(
        reverse(
            "raport_slotow:raportslotowuczelnia-regen",
            args=(rsu.pk,),
        )
    )

    rsu.refresh_from_db()
    assert rsu.finished_on != first_finished_on



import json

import pytest
from django.urls import reverse

from fixtures import pbn_publication_json

from bpp.models import Autor_Dyscyplina, Uczelnia
from bpp.views.api import OstatniaJednostkaIDyscyplinaView, const
from bpp.views.api.pbn_get_by_parameter import GetPBNPublicationsByISBN
from bpp.views.api.pubmed import GetPubmedIDView, get_data_from_ncbi


def test_get_data_from_ncbi(mocker):
    query = mocker.patch("pymed.PubMed.query")
    query.return_value = "123"

    res = get_data_from_ncbi("foobar")
    assert res == ["1", "2", "3"]


def test_GetPubmedIDView_post_nie_ma_tytulu(rf):
    v = GetPubmedIDView()

    req = rf.get("/", data={"t": ""})
    res = json.loads(v.post(req).content)
    assert res == dict(error=const.PUBMED_BRAK_PARAMETRU)

    req = rf.get("/", data={"t": "    "})
    res = json.loads(v.post(req).content)
    assert res == dict(error=const.PUBMED_BRAK_PARAMETRU)


def test_GetPubmedIDView_post_brak_rezultaut(rf, mocker):

    query = mocker.patch("pymed.PubMed.query")
    query.return_value = []

    v = GetPubmedIDView()

    req = rf.post("/", data={"t": "razd dwa trzy test"})
    res = json.loads(v.post(req).content)
    assert res == dict(error=const.PUBMED_PO_TYTULE_BRAK)


def test_GetPubmedIDView_post_wiele_rezultatow(rf, mocker):

    query = mocker.patch("pymed.PubMed.query")
    query.return_value = ["1", "2", "3"]

    v = GetPubmedIDView()

    req = rf.post("/", data={"t": "razd dwa trzy test"})
    res = json.loads(v.post(req).content)
    assert res == dict(error=const.PUBMED_PO_TYTULE_WIELE)


def test_GetPubmedIDView_post_jeden_rezultat(rf, mocker):
    class FakePraca:
        pubmed_id = "lel"
        doi = "lol"
        pmc_id = "pmc_id"
        title = "none"

    query = mocker.patch("pymed.PubMed.query")
    query.return_value = [FakePraca()]

    v = GetPubmedIDView()

    req = rf.post("/", data={"t": "razd dwa trzy test"})
    res = json.loads(v.post(req).content)
    assert res["doi"] == "lol"


@pytest.mark.django_db
def test_GetPBNPublicationsByISBN_jedna_praca(
    rf, pbn_uczelnia, pbn_client, admin_user, wydawnictwo_nadrzedne
):
    ROK = 123
    ISBN = "123"
    UID_REKORDU = "foobar"
    TYTUL_REKORDU = "Jakis tytul"

    orig = Uczelnia.objects.get_default

    pub1 = pbn_publication_json(
        mongoId=UID_REKORDU, year=ROK, isbn=ISBN, title=TYTUL_REKORDU
    )

    wydawnictwo_nadrzedne.isbn = ISBN
    wydawnictwo_nadrzedne.rok = ROK
    wydawnictwo_nadrzedne.tytul_oryginalny = TYTUL_REKORDU
    wydawnictwo_nadrzedne.save()

    req = rf.post("/", data=dict(t=ISBN, rok="2021"))
    req.user = admin_user

    pbn_client.transport.return_values["/api/v1/search/publications?size=10"] = [pub1]
    pbn_client.transport.return_values[f"/api/v1/publications/id/{UID_REKORDU}"] = pub1
    try:
        Uczelnia.objects.get_default = lambda *args, **kw: pbn_uczelnia

        res = GetPBNPublicationsByISBN(request=req).post(req)
    finally:
        Uczelnia.objects.get_default = orig

    assert json.loads(res.content)["id"] == UID_REKORDU


@pytest.mark.django_db
def test_GetPBNPublicationsByISBN_wiele_isbn(
    rf, pbn_uczelnia, pbn_client, admin_user, wydawnictwo_nadrzedne
):
    ROK = 123
    ISBN = "123"
    UID_REKORDU = "foobar"
    TYTUL_REKORDU = "Jakis tytul"

    orig = Uczelnia.objects.get_default

    pub1 = pbn_publication_json(
        mongoId=UID_REKORDU, year=ROK, isbn=ISBN, title=TYTUL_REKORDU
    )
    pub2 = pbn_publication_json(
        mongoId=UID_REKORDU + "2", year=ROK, isbn=ISBN, title=TYTUL_REKORDU
    )

    wydawnictwo_nadrzedne.isbn = ISBN
    wydawnictwo_nadrzedne.rok = ROK
    wydawnictwo_nadrzedne.tytul_oryginalny = TYTUL_REKORDU
    wydawnictwo_nadrzedne.save()

    req = rf.post("/", data=dict(t=ISBN, rok="2021"))
    req.user = admin_user

    pbn_client.transport.return_values["/api/v1/search/publications?size=10"] = [
        pub1,
        pub2,
    ]
    pbn_client.transport.return_values[f"/api/v1/publications/id/{UID_REKORDU}"] = pub1
    pbn_client.transport.return_values[f"/api/v1/publications/id/{UID_REKORDU}2"] = pub2

    try:
        Uczelnia.objects.get_default = lambda *args, **kw: pbn_uczelnia

        res = GetPBNPublicationsByISBN(request=req).post(req)
    finally:
        Uczelnia.objects.get_default = orig

    assert json.loads(res.content)["id"] == UID_REKORDU


@pytest.fixture
def ojv():
    return OstatniaJednostkaIDyscyplinaView()


def test_ostatnia_jednostka_view(autor, jednostka, rf, ojv, dyscyplina1):
    jednostka.dodaj_autora(autor)

    Autor_Dyscyplina.objects.create(
        rok=2000, autor=autor, dyscyplina_naukowa=dyscyplina1
    )

    fr = rf.post("/", data=dict(autor_id=autor.pk, rok=2000))
    res = ojv.post(fr)
    res = json.loads(res.content)
    assert "jednostka_id" in res
    assert "dyscyplina_id" in res


def test_ostatnia_jednostka_view_dwie_dysc(
    autor, jednostka, rf, ojv, dyscyplina1, dyscyplina2
):
    jednostka.dodaj_autora(autor)

    Autor_Dyscyplina.objects.create(
        rok=2000,
        autor=autor,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    fr = rf.post("/", data=dict(autor_id=autor.pk, rok=2000))
    res = ojv.post(fr)
    res = json.loads(res.content)
    assert "jednostka_id" in res
    assert "dyscyplina_id" not in res


def test_ostatnia_jednostka_view_rok_empty(autor, jednostka, rf, ojv):
    jednostka.dodaj_autora(autor)

    fr = rf.post("/", data=dict(autor_id=autor.pk, rok=""))
    res = ojv.post(fr)
    res = json.loads(res.content)
    assert "jednostka_id" in res
    assert "dyscyplina_id" not in res


def test_ostatnia_jednostka_errors_autor_null(rf, ojv):
    fr = rf.post("/", data=dict(autor_id=""))
    res = ojv.post(fr)
    assert json.loads(res.content)["status"] == "error"


@pytest.mark.django_db
def test_ostatnia_jednostka_errors_autor_404(rf, ojv):
    fr = rf.post("/", data=dict(autor_id=10))
    res = ojv.post(fr)
    assert json.loads(res.content)["status"] == "error"


@pytest.mark.django_db
def test_ostatnia_jednostka_errors_autor_404_2(rf, ojv):
    fr = rf.post("/", data=dict(autor_id="heyyy"))
    res = ojv.post(fr)
    assert json.loads(res.content)["status"] == "error"


def test_ostatnia_jednostka_errors_autor_bez_dysc(rf, ojv, autor):
    fr = rf.post("/", data=dict(autor_id=autor.pk))
    res = ojv.post(fr)

    res = json.loads(res.content)
    assert res["jednostka_id"] is None
    assert res["nazwa"] is None
    assert res["status"] == "ok"


def test_ustaw_orcid_autora(csrf_exempt_django_admin_app, autor_jan_kowalski):
    ORCID = "1" * 19

    assert autor_jan_kowalski.orcid != ORCID
    csrf_exempt_django_admin_app.post(
        reverse("bpp:api_ustaw_orcid"),
        params={"autor": autor_jan_kowalski.pk, "orcid": ORCID},
    )

    autor_jan_kowalski.refresh_from_db()
    assert autor_jan_kowalski.orcid == ORCID



import json

from pbn_api.adapters.wydawnictwo import WydawnictwoPBNAdapter
from pbn_api.management.commands.util import PBNBaseCommand

from django.contrib.contenttypes.models import ContentType


class Command(PBNBaseCommand):
    help = "pokaż kod JSON wygenerowany dla publikacji"

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument("model_name")
        parser.add_argument("id", type=int)

    def handle(self, model_name, id, *args, **kw):
        obj = ContentType.objects.get(
            app_label="bpp", model=model_name
        ).get_object_for_this_type(id=id)
        adapted = WydawnictwoPBNAdapter(obj)
        data = adapted.pbn_get_json()
        print(json.dumps(data, indent=4, sort_keys=True))



# -*- encoding: utf-8 -*-
import json

import pytest
from django import forms

from bpp.views.api.uzupelnij_rok import ApiUzupelnijRokWydawnictwoZwarteView, \
    ApiUzupelnijRokWydawnictwoCiagleView


@pytest.mark.django_db
def test_ApiUzupelnijRokWydawnictwoZwarteView_get_data(wydawnictwo_zwarte):
    x = ApiUzupelnijRokWydawnictwoZwarteView()

    test_dict_1 = {'miejsce_i_rok': "Lublin 2000"}
    test_dict_2 = {'wydawnictwo_nadrzedne': wydawnictwo_zwarte}
    res = x.get_data(test_dict_1)
    assert res['rok'] == "2000"

    res = x.get_data(test_dict_2)
    assert res['rok'] == wydawnictwo_zwarte.rok

    test_dict_1.update(test_dict_2)
    res = x.get_data(test_dict_1)

    assert res['rok'] == "2000"


def test_ApiUzupelnijRokWydawnictwoZwarteView_post(rf):
    x = ApiUzupelnijRokWydawnictwoZwarteView()

    class FakeForm(forms.Form):
        sss = forms.CharField(required=True)

    x.validation_form_class = FakeForm

    res = x.post(rf.post("/"))
    assert json.loads(res.content)['error'] == 'form'

    res = x.post(rf.post("/", data={"sss": "123"}))
    assert json.loads(res.content)['rok'] == 'b/d'


def test_ApiUzupelnijRokWydawnictwoCiagleView():
    x = ApiUzupelnijRokWydawnictwoCiagleView()

    test_dict_1 = {'informacje': "Lublin 2000"}
    res = x.get_data(test_dict_1)
    assert res['rok'] == "2000"

    res = x.get_data({'informacje': ''})
    assert res['rok'] == "b/d"

    res = x.get_data({'informacje': 'Lublni 202'})
    assert res['rok'] == "b/d"



# Generated by Django 3.2.15 on 2022-10-02 23:13

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [
        ("dynamic_columns", "0001_initial"),
        ("dynamic_columns", "0002_auto_20221002_2252"),
        ("dynamic_columns", "0003_modeladmin_model_ref"),
    ]

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="ModelAdmin",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("class_name", models.TextField()),
                (
                    "model_ref",
                    models.ForeignKey(
                        default=1,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ModelAdminColumn",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("col_name", models.CharField(max_length=255)),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="dynamic_columns.modeladmin",
                    ),
                ),
                ("enabled", models.BooleanField(default=True)),
                ("ordering", models.PositiveSmallIntegerField(default=1)),
            ],
            options={
                "unique_together": {("parent", "col_name")},
                "ordering": ("parent", "ordering"),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-19 12:40
from __future__ import unicode_literals

import bpp.models.cache
from decimal import Decimal
import django.contrib.postgres.fields
import django.contrib.postgres.search
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0108_bpp_nowe_sumy_view_bug'),
    ]

    operations = [
        migrations.CreateModel(
            name='RekordView',
            fields=[
                ('opis_bibliograficzny_cache', models.TextField(default='')),
                ('opis_bibliograficzny_autorzy_cache', django.contrib.postgres.fields.ArrayField(base_field=models.TextField(), blank=True, null=True, size=None)),
                ('opis_bibliograficzny_zapisani_autorzy_cache', models.TextField(default='')),
                ('liczba_znakow_wydawniczych', models.IntegerField(blank=True, db_index=True, null=True, verbose_name='Liczba znaków wydawniczych')),
                ('rok', models.IntegerField(db_index=True, help_text='Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)')),
                ('recenzowana', models.BooleanField(db_index=True, default=False)),
                ('impact_factor', models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'), max_digits=6)),
                ('punkty_kbn', models.DecimalField(db_index=True, decimal_places=2, default=Decimal('0.00'), max_digits=6, verbose_name='Punkty KBN')),
                ('index_copernicus', models.DecimalField(db_index=True, decimal_places=2, default=Decimal('0.00'), max_digits=6, verbose_name='Index Copernicus')),
                ('punktacja_wewnetrzna', models.DecimalField(db_index=True, decimal_places=2, default=Decimal('0.00'), max_digits=6, verbose_name='Punktacja wewnętrzna')),
                ('kc_impact_factor', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, help_text='Jeżeli wpiszesz\n        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji\n        Centralnej w punkcie IXa tego raportu.', max_digits=6, null=True, verbose_name='KC: Impact factor')),
                ('kc_punkty_kbn', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, help_text='Jeżeli wpiszesz\n        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji\n        Centralnej w punkcie IXa i IXb tego raportu.', max_digits=6, null=True, verbose_name='KC: Punkty KBN')),
                ('kc_index_copernicus', models.DecimalField(blank=True, decimal_places=2, default=None, help_text='Jeżeli wpiszesz\n        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji\n        Centralnej w punkcie IXa i IXb tego raportu.', max_digits=6, null=True, verbose_name='KC: Index Copernicus')),
                ('informacje', models.TextField(blank=True, db_index=True, null=True, verbose_name='Informacje')),
                ('szczegoly', models.CharField(blank=True, help_text='Np. str. 23-45', max_length=512, null=True, verbose_name='Szczegóły')),
                ('uwagi', models.TextField(blank=True, db_index=True, null=True)),
                ('slowa_kluczowe', models.TextField(blank=True, null=True, verbose_name='Słowa kluczowe')),
                ('utworzono', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Utworzono')),
                ('openaccess_ilosc_miesiecy', models.PositiveIntegerField(blank=True, help_text='Ilość miesięcy jakie upłynęły od momentu opublikowania do momentu udostępnienia', null=True, verbose_name='OpenAccess: ilość miesięcy')),
                ('id', bpp.models.cache.TupleField(base_field=models.IntegerField(), primary_key=True, serialize=False, size=2)),
                ('tytul_oryginalny', models.TextField()),
                ('tytul', models.TextField()),
                ('search_index', django.contrib.postgres.search.SearchVectorField()),
                ('wydawnictwo', models.TextField()),
                ('adnotacje', models.TextField()),
                ('ostatnio_zmieniony', models.DateTimeField()),
                ('tytul_oryginalny_sort', models.TextField()),
                ('www', models.URLField(blank=True, max_length=1024, null=True, verbose_name='Adres WWW')),
            ],
            options={
                'db_table': 'bpp_rekord',
                'managed': False,
            },
        ),
    ]



# -*- encoding: utf-8 -*-
from bpp.models.abstract import ModelZNazwa, ModelZISSN


class Seria_Wydawnicza(ModelZNazwa):
    class Meta:
        verbose_name_plural = "serie wydawnicze"
        verbose_name = "seria wydawnicza"
        ordering = ('nazwa',)
    pass
